static const int MAX_LIGHT_COUNT = 2;
static const int MAX_ANISOTROPY = 16;

matrix WV;
matrix WVP;

vector LightAmbient;

int LightCountPoint;
float3 LightPosPoint[MAX_LIGHT_COUNT];
float3 LightColorPoint[MAX_LIGHT_COUNT];
float LightRangePoint[MAX_LIGHT_COUNT];

int LightCountDirectional;
float3 LightDirDirectional[MAX_LIGHT_COUNT];
float3 LightColorDirectional[MAX_LIGHT_COUNT];

texture TexDiffuse;

sampler SampDiffuse = sampler_state {
    Texture = <TexDiffuse>;
	MaxAnisotropy = MAX_ANISOTROPY;
    MinFilter = Anisotropic;  
    MagFilter = Anisotropic;
    MipFilter = Anisotropic;
    AddressU  = Wrap;
    AddressV  = Wrap;
};

struct TVSInput {
	float3 _Position0: Position0;
	float3 _Normal0: Normal0;
	float4 _Diffuse0: Diffuse0;
	float2 _TexCoord0: TexCoord0;
};

struct TPSInput {
	float4 _Position0: Position0;
	float2 _TexCoord0: TexCoord0;
	float3 _WorldPosition: TexCoord1;
	float3 _WorldNormal: TexCoord2;
};

void VShader (const in TVSInput Input, out TPSInput Output) {
	Output._Position0 = mul(float4(Input._Position0, 1), WVP);
	Output._TexCoord0 = (float2)Input._TexCoord0;
	Output._WorldPosition = mul(float4(Input._Position0, 1), (float4x3)WV);
	Output._WorldNormal = mul(Input._Normal0, (float3x3)WV);
}

float4 PShader (const in TPSInput Input): Color0 {
	float3 CamDir = normalize(Input._WorldPosition);
	float3 Normal = normalize(Input._WorldNormal);
	float3 RefDir = CamDir - 2 * Normal * dot(CamDir, Normal);
	float3 Dif = (0, 0, 0);
	float3 Spec = (0, 0, 0);
	float Att = 0;
	float3 VertexToLight;
	for (int i = 0; i < LightCountPoint; i++) {
		VertexToLight = LightPosPoint[i] - Input._WorldPosition;
		Att = clamp(1 - dot(VertexToLight, VertexToLight) / LightRangePoint[i], 0, 1);
		VertexToLight = normalize(VertexToLight);
		Dif += saturate(dot(VertexToLight, Normal) * LightColorPoint[i]) * Att;
		Spec += saturate(dot(VertexToLight, RefDir) * LightColorPoint[i]) * Att;
	}
	Dif += saturate(dot(LightDirDirectional[0], Normal) * LightColorDirectional[0]); 
	Spec += saturate(dot(LightDirDirectional[0], RefDir) * LightColorDirectional[0]); 
	float4 TexDiffuse = tex2D(SampDiffuse, Input._TexCoord0);
	float4 Output = TexDiffuse * float4(saturate(Dif) + LightAmbient, 1) + float4(pow(saturate(Spec), 10) * 0.5, 0);
	return Output;
}

technique SpecularLighting {
	pass Pass0 {
		VertexShader = compile vs_2_0 VShader();
		PixelShader = compile ps_2_0 PShader();
	}
}