program Arcanoid;

uses
  Windows,
  SysUtils,
  Messages,
  Classes,
  Gen2,
  G2Math,
  G2Group,
  G2MeshLoaderG2M,
  Direct3D9,
  D3DX9,
  DXTypes,
  Types,
  Math,
  DirectInput,
  CSD;

type
  PGroup = ^TGroup;
  TGroup = record
    Pos1, Pos2: TG2Vec2;
    Pivot: TG2Vec2;
    VelAng: Single;
    Angle: Single;
    Mat3D: TG2Mat;
    Mat2D: TG2Mat;
    Blocks: TList;
    VB: TG2VB;
    IB: TG2IB;
    VBFrameInd: DWord;
    IBFrameInd: DWord;
    FrameVCount: DWord;
    FrameICount: DWord;
    ExterVCount: DWord;
    ExterICount: DWord;
    SubsetCount: DWord;
    Subsets: array of record
      VBInd, IBInd: DWord;
      VCount, ICount: DWord;
      TexID: Integer;
    end;
    procedure AddSubset(const SubVBInd, SubIBInd, SubVCount, SubICount: DWord; const SubTexID: Integer);
    procedure Sort;
    procedure Build;
  end;

  PBlock = ^TBlock;
  TBlock = record
    p: array of TG2Vec2;
    Poly: TCSPolygon;
    Bonds: TG2Rect;
    Group: PGroup;
    DieMat: TG2Mat;
    DieTime: DWord;
    Dead: Boolean;
    Level: Integer;
    Frame: record
      Vec: array of TG2Vec3;
      Nrm: array of TG2Vec3;
      Tex: array of TG2Vec2;
      Ind: array of Word;
    end;
    Exter: record
      Vec: array of TG2Vec3;
      Nrm: array of TG2Vec3;
      Tex: array of TG2Vec2;
      Ind: array of Word;
    end;
    procedure BuildFrame;
    procedure BuildExter;
  end;

  TMap = record
    BAreaW, BAreaH: Single;
    W, H: Single;
  end;

  TBall = class (TG2GroupItem)
  public
    R: Single;
    V: TG2Vec2;
    Inst: TG2MeshInstance;
    m: TG2Mat;
    Rot: TG2Quat;
    Pos: TG2Vec3;
    Light: Integer;
    Fixed: Boolean;
    FixedPos: Single;
    constructor Create(const Group: TG2Group); override;
    destructor Destroy; override;
    procedure Render; override;
    procedure Update; override;
    function Proximity(const Block: PBlock): Boolean;
  end;

  TBar = record
    Pos: Single;
    Len: Single;
  end;

  TMyApp = class (TG2App)
  strict private
    m_Running: Boolean;
  public
    MgrTextures: TG2TextureMgr;
    MgrMeshes: TG2MeshMgr;
    MgrEffects: TG2EffectMgr;
    TexFloor: TG2Texture2D;
    TexFloorNM: TG2Texture2D;
    TexFloorSM: TG2Texture2D;
    TexColumn: TG2Texture2D;
    TexColumnNM: TG2Texture2D;
    TexColumnSM: TG2Texture2D;
    TexWall: TG2Texture2D;
    TexWallNM: TG2Texture2D;
    TexWallSM: TG2Texture2D;
    TexBorder: TG2Texture2D;
    TexBall: TG2Texture2D;
    TexLife: TG2Texture2D;
    TexBlocks: array [0..3] of TG2Texture2D;
    MgrSounds: TG2SoundMgr;
    SndMusic1: TG2Sound;
    SndInstMusic1: TG2SoundInst;
    SndBell1: TG2Sound;
    SndInstBell1: TG2SoundInst;
    SndBell2: TG2Sound;
    SndInstBell2: TG2SoundInst;
    SndBell3: TG2Sound;
    SndInstBell3: TG2SoundInst;
    SndBallLost: TG2Sound;
    SndInstBallLost: TG2SoundInst;
    FxLighting: TG2Effect;
    MshField: TG2Mesh;
    MshInstField: TG2MeshInstance;
    MshBall: TG2Mesh;
    MshInstBall: TG2MeshInstance;
    Font: TG2Font;
    Map: TMap;
    Blocks: TList;
    Groups: TList;
    GroupZero: PGroup;
    DieBlocks: TList;
    RemBlocks: TList;
    Balls: TG2Group;
    Bar: TBar;
    InitBlocks: Integer;
    Lives: Integer;
    HiSpecs: Boolean;
    DefMeshMode: TG2MeshMode;
    StartNextLevel: Boolean;
    NextLevelTime: DWord;
    CurLevel: Integer;
    FinishScreen: Boolean;
    GameOver: Boolean;
    MPlayTime: DWord;
    property Running: Boolean read m_Running;
    procedure Initialize; override;
    procedure Finalize; override;
    procedure OnTimer; override;
    procedure OnRender; override;
    procedure OnUpdate; override;
    procedure OnKeyDown(const Key: Byte); override;
    procedure LoadBlocks(const f: AnsiString);
    procedure DrawBlocks;
    procedure FreeBlocks;
    procedure FreeGroups;
    procedure AddBall(const Vel: Single);
    function CheckComplete: Boolean;
    procedure WrapUp;
  end;

var
  App: TMyApp;

const
  BLOCK_DIE_TIME = 300;

//TMyApp BEGIN
procedure TMyApp.Initialize;
var
  i, sw, sh: Integer;
begin
  Randomize;
  ShowCursor(False);
  m_Running := True;
  Gfx.InitParams.FastPresent := True;
  sw := GetSystemMetrics(SM_CXSCREEN);
  sh := GetSystemMetrics(SM_CYSCREEN);
  sh := Trunc(1024 * (sh / sw));
  sw := 1024;
  Gfx.InitParams.Width := sw;
  Gfx.InitParams.Height := sh;
  Gfx.InitParams.DepthStencilFormat := D3DFMT_D16;
  Tmr.Mode := tmManual;
  Tmr.MaxFPS := 0;
  Tmr.TargetUPS := 200;
  inherited Initialize;
  if D3DSHADER_VERSION_MAJOR(Gfx.Caps.PixelShaderVersion) >= 3 then
  begin
    HiSpecs := True;
    DefMeshMode := mmSM3HLSL;
  end
  else
  begin
    if D3DSHADER_VERSION_MAJOR(Gfx.Caps.PixelShaderVersion) >= 2 then
    HiSpecs := True
    else
    HiSpecs := False;
    DefMeshMode := mmFF;
  end;
  g2.RequestMod(TG2TextureMgr, @MgrTextures);
  g2.RequestMod(TG2MeshMgr, @MgrMeshes);
  g2.RequestMod(TG2EffectMgr, @MgrEffects);
  g2.RequestMod(TG2SoundMgr, @MgrSounds);
  TexFloor := MgrTextures.CreateTexture2DFromFile('Floor', 'gfx\Floor2.jpg');
  TexFloorNM := MgrTextures.CreateTexture2DFromFile('FloorNM', 'gfx\Floor2NM.jpg');
  TexFloorSM := MgrTextures.CreateTexture2DFromFile('FloorSM', 'gfx\Floor2SM.jpg');
  TexColumn := MgrTextures.CreateTexture2DFromFile('Column', 'gfx\Floor.jpg');
  TexColumnNM := MgrTextures.CreateTexture2DFromFile('ColumnNM', 'gfx\FloorNM.jpg');
  TexColumnSM := MgrTextures.CreateTexture2DFromFile('ColumnSM', 'gfx\FloorSM.jpg');
  TexWall := MgrTextures.CreateTexture2DFromFile('Wall', 'gfx\Wall.jpg');
  TexWallNM := MgrTextures.CreateTexture2DFromFile('WallNM', 'gfx\WallNM.jpg');
  TexWallSM := MgrTextures.CreateTexture2DFromFile('WallSM', 'gfx\WallSM.jpg');
  TexBorder := MgrTextures.CreateTexture2DFromFile('Border', 'gfx\Border.png', 1);
  TexBall := MgrTextures.CreateTexture2DFromFile('Ball', 'gfx\Ball.jpg', 16);
  for i := 0 to 3 do
  TexBlocks[i] := MgrTextures.CreateTexture2DFromFile('Block' + IntToStr(i), 'gfx\Block' + IntToStr(i) + '.jpg');
  TexLife := MgrTextures.CreateTexture2DFromFile('Life', 'gfx\Ball.png', 1);
  FxLighting := MgrEffects.CreateEffectFromFile('Lighting', 'Lighting.fx');
  MshField := MgrMeshes.CreateMeshFromFile('Field', 'gfx\Field.g2m', DefMeshMode);
  MshField.Materials[1].Diffuse := TexFloor;
  MshField.Materials[1].Specular := TexFloorSM;
  MshField.Materials[1].Normals := TexFloorNM;
  MshField.Materials[0].Diffuse := TexColumn;
  MshField.Materials[0].Specular := TexColumnSM;
  MshField.Materials[0].Normals := TexColumnNM;
  MshField.Materials[2].Diffuse := TexWall;
  MshField.Materials[2].Specular := TexWallSM;
  MshField.Materials[2].Normals := TexWallNM;
  MshInstField := MshField.CreateInstance;
  MshBall := MgrMeshes.CreateMeshFromFile('Ball', 'gfx\Ball.g2m');
  MshBall.Materials[0].Diffuse := TexBall;
  MshInstBall := MshBall.CreateInstance;
  Font := Gfx.Shared.RequestFont('Arial', 16);
  SndMusic1 := MgrSounds.CreateSound('Music');
  SndMusic1.LoadFile('music0.mid');
  SndInstMusic1 := SndMusic1.CreateInstance;
  SndInstMusic1.Play;
  SndBell1 := MgrSounds.CreateSound('Bell');
  SndBell1.LoadFile('bell.wav');
  SndInstBell1 := SndBell1.CreateInstance;
  SndBell2 := MgrSounds.CreateSound('Bell2');
  SndBell2.LoadFile('bell2.wav');
  SndInstBell2 := SndBell2.CreateInstance;
  SndBell3 := MgrSounds.CreateSound('Bell3');
  SndBell3.LoadFile('bell3.wav');
  SndInstBell3 := SndBell3.CreateInstance;
  SndBallLost := MgrSounds.CreateSound('BallLost');
  SndBallLost.LoadFile('balllost.wav');
  SndBallLost.PlayRate := 4;
  SndInstBallLost := SndBallLost.CreateInstance;
  MPlayTime := AppTime;
  Map.W := 20;
  Map.H := 20;
  Map.BAreaW := Map.W;
  Map.BAreaH := Map.W * (5 / 4);
  Groups := TList.Create;
  Blocks := TList.Create;
  DieBlocks := TList.Create;
  RemBlocks := TList.Create;
  CurLevel := 1;
  FinishScreen := False;
  GameOver := False;
  Balls := TG2Group.Create;
  Gfx.Lights.Lights[0].SetDirectionalLight(1, -6, 8, $ffffffff);
  Gfx.Lights.Lights[0].Enabled := True;
  Gfx.Lights.Lights[0].SetToDevice;
  Gfx.SetMaterialDefault;
  Gfx.RenderStates.Lighting := True;
  Bar.Len := 4;
  Bar.Pos := Map.W * 0.5;
  g2.Timer.Enabled := True;
  LoadBlocks('map1.map');
  AddBall(0.05);
end;

procedure TMyApp.Finalize;
begin
  Balls.FreeItems;
  Balls.Free;
  FreeBlocks;
  RemBlocks.Free;
  DieBlocks.Free;
  Blocks.Free;
  FreeGroups;
  Groups.Free;
  inherited Finalize;
end;

procedure TMyApp.OnTimer;
begin
  if not SndInstMusic1.IsPlaying
  and (AppTime - MPlayTime > 1000) then
  begin
    SndInstMusic1.Play;
    MPlayTime := AppTime;
  end;
end;

procedure TMyApp.OnRender;
var
  i, j: Integer;
  tr: TG2Rect;
  m: TG2Mat;
  Dir: TG2Vec3;
  g, lx, ly, lz: Single;
  str: AnsiString;
begin
  Render.Clear(False, True, True, $ff808080);
  Render.RenderStart;

  if not FinishScreen then
  begin
    if InitBlocks > 0 then
    begin
      g := 1 - (Blocks.Count / InitBlocks);
      g := 180 + g * 75;
      Gfx.Lights.Lights[0].SetDirectionalLight((Bar.Pos - 10) * 0.1, -10, 10, G2Color(Trunc(g), Trunc(g), Trunc(g)));
      Gfx.Lights.Lights[0].Enabled := True;
      Gfx.Lights.Lights[0].SetToDevice;
    end;

    Gfx.Lights[1].SetPointLight(Bar.Pos, 1, 1, $ff888888, 10);
    Gfx.Lights[1].Enabled := True;
    Gfx.Lights[1].SetToDevice;

    Cam.SetView(10, 25, 0, 10, 0, 11, 0, 1, 0);
    Gfx.Transforms.P := Cam.Proj;
    Gfx.Transforms.V := Cam.View;
    Gfx.Transforms.ApplyP;
    Gfx.Transforms.ApplyV;

    RenderModes.FilteringAnisotropic(0, 16);
    Gfx.RenderStates.ZEnable := True;

    Balls.Render;

    DrawBlocks;

    m.SetTranslation(10, 0, 10);
    MshInstField.Transform := m;
    MshInstField.Render;

    Gfx.RenderStates.ZEnable := False;

    Render.TextureClear(0);
    Gfx.Transforms.PushW;
    m.SetRotationZ(HalfPi);
    m.Translate(Bar.Pos, 0, 1);
    Gfx.Transforms.W[0] := m; Gfx.Transforms.ApplyW(0);
    Prim3D.DrawCapsule(0, 0, 0, Bar.Len - 0.6, 0.3, $ffffffff);
    Gfx.Transforms.PopW;
    Gfx.Transforms.ApplyW(0);

    RenderModes.FilteringPoint();

    for i := 0 to Lives - 1 do
    Render2D.DrawRect(
      i * 40, 0, 64, 64, $ffffffff, TexLife
    );

    if StartNextLevel then
    begin
      str := 'Level Complete!';
      Font.Print(
        (Gfx.Params.Width - Font.GetTextWidth(str)) * 0.5,
        (Gfx.Params.Height - Font.GetTextHeight(str)) * 0.5 - 16,
        $ffffffff, str
      );
    end;

    if GameOver then
    begin
      str := 'Game Over!';
      Font.Print(
        (Gfx.Params.Width - Font.GetTextWidth(str)) * 0.5,
        (Gfx.Params.Height - Font.GetTextHeight(str)) * 0.5 - 16,
        $ffffffff, str
      );
    end;

  end
  else
  begin
    str := 'Congratulations!';
    Font.Print(
      (Gfx.Params.Width - Font.GetTextWidth(str)) * 0.5,
      (Gfx.Params.Height - Font.GetTextHeight(str)) * 0.5 - 16,
      $ffffffff, str
    );
    str := 'You have finished the game=)';
    Font.Print(
      (Gfx.Params.Width - Font.GetTextWidth(str)) * 0.5,
      (Gfx.Params.Height - Font.GetTextHeight(str)) * 0.5 + 16,
      $ffffffff, str
    );
  end;

  //Font.Print(10, 10, $ffffffff, 'FPS: ' + IntToStr(Tmr.FPS));

  Render.RenderStop;
  Render.Present;
end;

procedure TMyApp.OnUpdate;
var
  i, b, j: Integer;
  pb: PBlock;
  pg: PGroup;
  v: TG2Vec2;
  m: TG2Mat;
  Pos: TG2Vec2;
  g: Single;
begin
  if not StartNextLevel and not GameOver then
  begin
    for i := 0 to Groups.Count - 1 do
    begin
      pg := PGroup(Groups[i]);
      g := Sin(G2TimeInterval(10000) * TwoPi) * 0.5 + 0.5;
      Pos := G2LerpVec2(pg^.Pos1, pg^.Pos2, g);
      pg^.Angle := pg^.Angle + pg^.VelAng;
      pg^.Mat3D.SetTranslation(G2Vec3(-pg^.Pivot.x, 0, -pg^.Pivot.y));
      pg^.Mat3D.RotateY(pg^.Angle);
      pg^.Mat3D.Translate(G2Vec3(pg^.Pivot.x, 0, pg^.Pivot.y));
      pg^.Mat3D.Translate(Pos.x, 0, Pos.y);
      pg^.Mat2D.SetTranslation(G2Vec3(-pg^.Pivot.x, -pg^.Pivot.y, 0));
      pg^.Mat2D.RotateZ(-pg^.Angle);
      pg^.Mat2D.Translate(G2Vec3(pg^.Pivot.x, pg^.Pivot.y, 0));
      pg^.Mat2D.Translate(Pos.x, Pos.y, 0);
    end;
    for i := 0 to Groups.Count - 1 do
    for b := 0 to PGroup(Groups[i])^.Blocks.Count - 1 do
    begin
      pg := PGroup(Groups[i]);
      pb := PBlock(pg^.Blocks[b]);
      for j := 0 to High(pb^.p) do
      begin
        v := pb^.p[j] * pg^.Mat2D;
        pb^.Poly[j].x := v.x;
        pb^.Poly[j].y := v.y;
      end;
      pb^.Bonds.Left := pb^.Poly[0].x;
      pb^.Bonds.Top := pb^.Poly[0].y;
      pb^.Bonds.Right := pb^.Poly[0].x;
      pb^.Bonds.Bottom := pb^.Poly[0].y;
      for j := 1 to High(pb^.p) do
      begin
        if pb^.Poly[j].x < pb^.Bonds.Left then pb^.Bonds.Left := pb^.Poly[j].x;
        if pb^.Poly[j].y < pb^.Bonds.Top then pb^.Bonds.Top := pb^.Poly[j].y;
        if pb^.Poly[j].x > pb^.Bonds.Right then pb^.Bonds.Right := pb^.Poly[j].x;
        if pb^.Poly[j].y > pb^.Bonds.Bottom then pb^.Bonds.Bottom := pb^.Poly[j].y;
      end;
    end;
    if PlugInput.KeyDown[DIK_LEFT] then Bar.Pos := Bar.Pos - 0.06;
    if PlugInput.KeyDown[DIK_RIGHT] then Bar.Pos := Bar.Pos + 0.06;
    if Bar.Pos + Bar.Len * 0.5 > Map.W then Bar.Pos := Map.W - Bar.Len * 0.5;
    if Bar.Pos - Bar.Len * 0.5 < 0 then Bar.Pos := Bar.Len * 0.5;
    Balls.Update;
    for i := 0 to DieBlocks.Count - 1 do
    begin
      pb := PBlock(DieBlocks[i]);
      if pb^.Dead then
      begin
        pb^.Dead := False;
        Blocks.Remove(pb);
      end;
      if AppTime - pb^.DieTime >= BLOCK_DIE_TIME then
      RemBlocks.Add(pb);
    end;
    for i := 0 to RemBlocks.Count - 1 do
    begin
      pb := PBlock(RemBlocks[i]);
      DieBlocks.Remove(pb);
      Dispose(pb);
    end;
    RemBlocks.Clear;

    if CheckComplete then
    begin
      StartNextLevel := True;
      NextLevelTime := AppTime;
      Inc(CurLevel);
    end;
    
    if (Balls.Count = 0) and (Lives = 0) then
    begin
      GameOver := True;
    end;
  end
  else
  begin
    if StartNextLevel and (AppTime - NextLevelTime > 3000) then
    begin
      if FileExists('map' + IntToStr(CurLevel) + '.map') then
      begin
        Tmr.Enabled := False;
        FreeBlocks;
        FreeGroups;
        Balls.FreeItems;
        StartNextLevel := False;
        LoadBlocks('map' + IntToStr(CurLevel) + '.map');
        AddBall(0.05);
        Tmr.Enabled := True;
      end
      else
      begin
        FinishScreen := True;
      end;
    end;
  end;
end;

procedure TMyApp.OnKeyDown(const Key: Byte);
var
  i: Integer;
  Vel, g: Single;
  m: TG2Mat;
begin
  if PlugInput.KeyDown[DIK_LCONTROL] and (Key = DIK_W) then
  begin
    for i := 0 to Blocks.Count - 1 do
    begin
      PBlock(Blocks[i])^.Group^.Blocks.Remove(PBlock(Blocks[i]));
      PBlock(Blocks[i])^.Group := nil;
      PBlock(Blocks[i])^.Dead := True;
      DieBlocks.Add(PBlock(Blocks[i]));
    end;
  end;
  if Key = DIK_SPACE then
  begin
    for i := 0 to Balls.Count - 1 do
    begin
      if TBall(Balls[i]).Fixed then
      begin
        TBall(Balls[i]).Fixed := False;
        Vel := TBall(Balls[i]).V.Len;
        g := (TBall(Balls[i]).Pos.x - (Bar.Pos - Bar.Len * 0.5)) / Bar.Len;
        g := G2LerpFloat(Pi - 0.3, 0.3, g);
        m.SetRotationZ(g);
        TBall(Balls[i]).V := (G2Vec2(1, 0) * m) * Vel;
      end;
    end;
  end;
  if (Key <> DIK_ESCAPE)
  and (GameOver or FinishScreen) then
  begin
    FreeBlocks;
    FreeGroups;
    Balls.FreeItems;
    CurLevel := 1;
    Lives := 3;
    LoadBlocks('map1.map');
    AddBall(0.05);
    GameOver := False;
    FinishScreen := False;
  end;
  if Key = DIK_ESCAPE then
  WrapUp;
end;

procedure TMyApp.LoadBlocks(const f: AnsiString);
var
  fs: TFileStream;
  i, j, b, nbr, PtsX, PtsY, gc, bc, pc: Integer;
  pb: PBlock;
  pg: PGroup;
  pt: TPoint;
begin
  FreeBlocks;
  FreeGroups;
  New(GroupZero);
  GroupZero^.Blocks := TList.Create;
  GroupZero^.VB := TG2VB.Create; GroupZero^.VB.Initialize(g2);
  GroupZero^.IB := TG2IB.Create; GroupZero^.IB.Initialize(g2);
  Groups.Add(GroupZero);
  GroupZero^.Pos1 := G2Vec2(0, 0);
  GroupZero^.Pos2 := G2Vec2(0, 0);
  GroupZero^.Pivot := G2Vec2(0, 0);
  GroupZero^.VelAng := 0;
  GroupZero^.Angle := 0;
  GroupZero^.Mat3D.SetIdentity;
  GroupZero^.Mat2D.SetIdentity;
  fs := TFileStream.Create(f, fmOpenRead);
  fs.Read(PtsX, 4);
  fs.Read(PtsY, 4);
  Map.BAreaH := Map.BAreaW * (PtsY / PtsX);
  //Inc(PtsX); Inc(PtsY);
  fs.Read(bc, 4);
  for i := 0 to bc - 1 do
  begin
    New(pb);
    fs.Read(pb^.Level, 4);
    fs.Read(pc, 4);
    SetLength(pb^.p, pc);
    SetLength(pb^.Poly, pc);
    for j := 0 to pc - 1 do
    begin
      fs.Read(pt, 8);
      pb^.p[j].x := (pt.x / PtsX) * Map.BAreaW;
      pb^.p[j].y := Map.H - (pt.y / PtsY) * Map.BAreaH;
      pb^.Poly[j].x := pb^.p[j].x; pb^.Poly[j].y := pb^.p[j].y;
    end;
    pb^.Group := nil;
    Blocks.Add(pb);
    pb^.Dead := False;
    pb^.BuildFrame;
    pb^.BuildExter;
    pb^.Bonds.Left := pb^.p[0].x;
    pb^.Bonds.Top := pb^.p[0].y;
    pb^.Bonds.Right := pb^.p[0].x;
    pb^.Bonds.Bottom := pb^.p[0].y;
    for j := 1 to pc - 1 do
    begin
      if pb^.p[j].x < pb^.Bonds.Left then pb^.Bonds.Left := pb^.p[j].x;
      if pb^.p[j].y < pb^.Bonds.Top then pb^.Bonds.Top := pb^.p[j].y;
      if pb^.p[j].x > pb^.Bonds.Right then pb^.Bonds.Right := pb^.p[j].x;
      if pb^.p[j].y > pb^.Bonds.Bottom then pb^.Bonds.Bottom := pb^.p[j].y;
    end;
  end;

  fs.Read(gc, 4);
  for i := 0 to gc - 1 do
  begin
    New(pg);
    pg^.Blocks := TList.Create;
    pg^.VB := TG2VB.Create; pg^.VB.Initialize(g2);
    pg^.IB := TG2IB.Create; pg^.IB.Initialize(g2);
    fs.Read(pg^.Pivot, 8);
    pg^.Pivot.x := (pg^.Pivot.x / PtsX) * Map.BAreaW;
    pg^.Pivot.y := Map.H - (pg^.Pivot.y / PtsY) * Map.BAreaH;
    fs.Read(j, 4);
    pg^.Angle := 0;
    pg^.VelAng := j / 1000;
    fs.Read(pt, 8);
    pg^.Pos1 := G2Vec2(0, 0);
    pg^.Pos2.x := (pt.x / PtsX) * Map.BAreaW;
    pg^.Pos2.y := (pt.y / PtsY) * Map.BAreaH;
    pg^.Mat3D.SetIdentity;
    pg^.Mat2D.SetIdentity;
    fs.Read(bc, 4);
    for b := 0 to bc - 1 do
    begin
      fs.Read(j, 4);
      PBlock(Blocks[j])^.Group := pg;
      pg^.Blocks.Add(PBlock(Blocks[j]));
    end;
    Groups.Add(pg);
  end;

  for i := 0 to Blocks.Count - 1 do
  begin
    pb := PBlock(Blocks[i]);
    if not Assigned(pb^.Group) then
    begin
      pb^.Group := GroupZero;
      GroupZero.Blocks.Add(pb);
    end;
  end;

  for i := 0 to Groups.Count - 1 do
  PGroup(Groups[i])^.Build;
  
  fs.Free;
  InitBlocks := Blocks.Count;
  Lives := 3;
  StartNextLevel := False;
end;

procedure TMyApp.DrawBlocks;
var
  i, j: Integer;
  pb: PBlock;
  pg: PGroup;
  v: TG2Vec3;
  m: TG2Mat;
  g: Single;
begin
  if HiSpecs then
  begin
    FxLighting.Technique := 'SpecularLighting';
    Gfx.Lights.SetToEffect(FxLighting);
    FxLighting.BeginEffect(nil);
    FxLighting.BeginPass(0);
  end;
  
  for i := 0 to Groups.Count - 1 do
  begin
    pg := PGroup(Groups[i]);
    pg^.VB.SetToDevice;
    pg^.IB.SetToDevice;
    Gfx.Transforms.W[0] := pg^.Mat3D;
    if HiSpecs then
    begin
      FxLighting.SetMatrix('WV', Gfx.Transforms.WV);
      FxLighting.SetMatrix('WVP', Gfx.Transforms.WVP);
    end
    else
    Gfx.Transforms.ApplyW(0);
    for j := 0 to pg^.SubsetCount - 1 do
    begin
      if HiSpecs then
      begin
        FxLighting.SetTexture('TexDiffuse', TexBlocks[pg^.Subsets[j].TexID].Texture);
        FxLighting.CommitChanges;
      end
      else
      Render.TextureSet(TexBlocks[pg^.Subsets[j].TexID]);
      Gfx.Device.DrawIndexedPrimitive(
        D3DPT_TRIANGLELIST,
        0, pg^.Subsets[j].VBInd, pg^.Subsets[j].VCount, pg^.Subsets[j].IBInd, pg^.Subsets[j].ICount div 3
      );
    end;
    if HiSpecs then
    begin
      FxLighting.SetTexture('TexDiffuse', TexBorder.Texture);
      FxLighting.CommitChanges;
    end
    else
    Render.TextureSet(TexBorder);
    Gfx.Device.DrawIndexedPrimitive(
      D3DPT_TRIANGLELIST,
      0, pg^.VBFrameInd, pg^.FrameVCount, pg^.IBFrameInd, pg^.FrameICount div 3
    );
  end;

  m.SetIdentity;
  Gfx.Transforms.W[0] := m;
  if HiSpecs then
  begin
    FxLighting.SetMatrix('WV', Gfx.Transforms.WV);
    FxLighting.SetMatrix('WVP', Gfx.Transforms.WVP);
  end
  else
  Gfx.Transforms.ApplyW(0);

  if DieBlocks.Count > 0 then
  begin
    Render3D.BaseVertexIndex := 0;
    Render3D.DrawBegin(ptTriangleList);
    for i := 0 to DieBlocks.Count - 1 do
    begin
      pb := PBlock(DieBlocks[i]);
      if HiSpecs then
      begin
        FxLighting.SetTexture('TexDiffuse', TexBlocks[pb^.Level].Texture);
        FxLighting.CommitChanges;
      end
      else
      Render.TextureSet(TexBlocks[pb^.Level]);
      g := Min(Max((AppTime - pb^.DieTime) / BLOCK_DIE_TIME, 0), 1);
      for j := 0 to High(pb^.Exter.Vec) do
      begin
        v := pb^.Exter.Vec[j] * pb^.DieMat;
        v.y := v.y - 2 * g;
        Render3D.AddPos(v);
        v := pb^.Exter.Nrm[j]; v.TransformNormal(pb^.DieMat);
        Render3D.AddNrm(v);
        Render3D.AddTex(pb^.Exter.Tex[j]);
      end;
      for j := 0 to High(pb^.Exter.Ind) do
      Render3D.AddInd(pb^.Exter.Ind[j]);
      Render3D.BaseVertexIndex := Render3D.BaseVertexIndex + Length(pb^.Exter.Vec);
    end;
    Render3D.DrawEnd;

    if HiSpecs then
    begin
      FxLighting.SetTexture('TexDiffuse', TexBorder.Texture);
      FxLighting.CommitChanges;
    end
    else
    Render.TextureSet(TexBorder);

    RenderModes.TexAddressClamp();
    Render3D.BaseVertexIndex := 0;
    Render3D.DrawBegin(ptTriangleList);
    for i := 0 to DieBlocks.Count - 1 do
    begin
      pb := PBlock(DieBlocks[i]);
      g := Min(Max((AppTime - pb^.DieTime) / BLOCK_DIE_TIME, 0), 1);
      for j := 0 to High(pb^.Frame.Vec) do
      begin
        v := pb^.Frame.Vec[j] * pb^.DieMat;
        v.y := v.y - 2 * g;
        Render3D.AddPos(v);
        v := pb^.Frame.Nrm[j]; v.TransformNormal(pb^.DieMat);
        Render3D.AddNrm(v);
        Render3D.AddTex(pb^.Frame.Tex[j]);
      end;
      for j := 0 to High(pb^.Frame.Ind) do
      Render3D.AddInd(pb^.Frame.Ind[j]);
      Render3D.BaseVertexIndex := Render3D.BaseVertexIndex + Length(pb^.Frame.Vec);
    end;
    Render3D.DrawEnd;
    Render3D.BaseVertexIndex := 0;
    RenderModes.TexAddressWrap();
  end;
  
  if HiSpecs then
  begin
    FxLighting.EndPass;
    FxLighting.EndEffect;
  end;
end;

procedure TMyApp.FreeBlocks;
var
  i: Integer;
begin
  for i := 0 to DieBlocks.Count - 1 do
  if not PBlock(DieBlocks[i])^.Dead then
  Dispose(PBlock(DieBlocks[i]));
  DieBlocks.Clear;
  for i := 0 to Blocks.Count - 1 do
  Dispose(PBlock(Blocks[i]));
  Blocks.Clear;
end;

procedure TMyApp.FreeGroups;
var
  i: Integer;
begin
  for i := 0 to Groups.Count - 1 do
  begin
    PGroup(Groups[i])^.IB.Free;
    PGroup(Groups[i])^.VB.Free;
    PGroup(Groups[i])^.Blocks.Free;
    Dispose(PGroup(Groups[i]));
  end;
  Groups.Clear;
end;

procedure TMyApp.AddBall(const Vel: Single);
begin
  with TBall.Create(Balls) do
  begin
    V.x := Vel;
  end;
end;

function TMyApp.CheckComplete: Boolean;
var
  i, j: Integer;
begin
  Result := True;
  for i := 0 to Groups.Count - 1 do
  for j := 0 to PGroup(Groups[i])^.Blocks.Count - 1 do
  begin
    if PBlock(PGroup(Groups[i])^.Blocks[j])^.Level > 0 then
    begin
      Result := False;
      Exit;
    end;
  end;
end;

procedure TMyApp.WrapUp;
begin
  m_Running := False;
end;
//TMyApp END

//TGroup BEGIN
procedure TGroup.AddSubset(const SubVBInd, SubIBInd, SubVCount, SubICount: DWord; const SubTexID: Integer);
begin
  if High(Subsets) < SubsetCount then
  begin
    if Length(Subsets) < 2 then
    SetLength(Subsets, 2)
    else
    SetLength(Subsets, Length(Subsets) * 2);
  end;
  Subsets[SubsetCount].VBInd := SubVBInd;
  Subsets[SubsetCount].IBInd := SubIBInd;
  Subsets[SubsetCount].VCount := SubVCount;
  Subsets[SubsetCount].ICount := SubICount;
  Subsets[SubsetCount].TexID := SubTexID;
  Inc(SubsetCount);
end;

procedure TGroup.Sort;
var
  i: Integer;
  sl: TG2SortedList;
  pb: PBlock;
begin
  sl := TG2SortedList.Create;
  for i := 0 to Blocks.Count - 1 do
  sl.Add(PBlock(Blocks[i]), PBlock(Blocks[i])^.Level);
  Blocks.Clear;
  for i := 0 to sl.Count - 1 do
  Blocks.Add(PBlock(sl.Items[i]));
  sl.Free;
end;

procedure TGroup.Build;
type
  TVertex = packed record
    Pos: TG2Vec3;
    Nrm: TG2Vec3;
    Tex: TG2Vec2;
  end;
  PVertexArray = ^TVertexArray;
  TVertexArray = array [0..0] of TVertex;
var
  i, j, vc, ic: Integer;
  pb: PBlock;
  CurV, CurI: DWord;
  Vertices: PVertexArray;
  Indices: PWordArray;
  SubVBInd, SubIBInd: DWord;
  SubTexID: Integer;
begin
  Sort;
  SubSetCount := 0;
  vc := 0; ic := 0;
  VBFrameInd := 0; IBFrameInd := 0;
  FrameVCount := 0; FrameICount := 0;
  ExterVCount := 0; ExterICount := 0;
  CurV := 0; CurI := 0;
  for i := 0 to Blocks.Count - 1 do
  begin
    pb := PBlock(Blocks[i]);
    Inc(vc, Length(pb^.Frame.Vec) + Length(pb^.Exter.Vec));
    Inc(ic, Length(pb^.Frame.Ind) + Length(pb^.Exter.Ind));
  end;
  VB.Verify(
    SizeOf(TVertex), vc,
    D3DUSAGE_WRITEONLY,
    D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_TEX1,
    D3DPOOL_MANAGED
  );
  IB.Verify(
    ic, D3DUSAGE_WRITEONLY,
    D3DFMT_INDEX16,
    D3DPOOL_MANAGED
  );
  VB.Lock(0, vc * SizeOf(TVertex), Pointer(Vertices), D3DLOCK_DISCARD);
  IB.Lock(0, ic * 2, Pointer(Indices), D3DLOCK_DISCARD);
  SubTexID := -1;
  SubVBInd := 0; SubIBInd := 0;
  for i := 0 to Blocks.Count - 1 do
  begin
    pb := PBlock(Blocks[i]);
    if SubTexID <> pb^.Level then
    begin
      if SubTexID >= 0 then
      begin
        AddSubset(SubVBInd, SubIBInd, CurV - SubVBInd, CurI - SubIBInd, SubTexID);
        SubVBInd := CurV;
        SubIBInd := CurI;
      end;
      SubTexID := pb^.Level;
    end;
    for j := 0 to High(pb^.Exter.Vec) do
    begin
      Vertices^[CurV + j].Pos := pb^.Exter.Vec[j];
      Vertices^[CurV + j].Nrm := pb^.Exter.Nrm[j];
      Vertices^[CurV + j].Tex := pb^.Exter.Tex[j];
    end;
    for j := 0 to High(pb^.Exter.Ind) do
    Indices^[CurI + j] := pb^.Exter.Ind[j] + CurV;
    Inc(CurV, Length(pb^.Exter.Vec));
    Inc(VBFrameInd, Length(pb^.Exter.Vec));
    Inc(ExterVCount, Length(pb^.Exter.Vec));
    Inc(CurI, Length(pb^.Exter.Ind));
    Inc(IBFrameInd, Length(pb^.Exter.Ind));
    Inc(ExterICount, Length(pb^.Exter.Ind));
  end;
  if (CurV > SubVBInd) and (CurI > SubIBInd) then
  AddSubset(SubVBInd, SubIBInd, CurV - SubVBInd, CurI - SubIBInd, SubTexID);
  for i := 0 to Blocks.Count - 1 do
  begin
    pb := PBlock(Blocks[i]);
    for j := 0 to High(pb^.Frame.Vec) do
    begin
      Vertices^[CurV + j].Pos := pb^.Frame.Vec[j];
      Vertices^[CurV + j].Nrm := pb^.Frame.Nrm[j];
      Vertices^[CurV + j].Tex := pb^.Frame.Tex[j];
    end;
    for j := 0 to High(pb^.Frame.Ind) do
    Indices^[CurI + j] := pb^.Frame.Ind[j] + CurV;
    Inc(CurV, Length(pb^.Frame.Vec));
    Inc(FrameVCount, Length(pb^.Frame.Vec));
    Inc(CurI, Length(pb^.Frame.Ind));
    Inc(FrameICount, Length(pb^.Frame.Ind));
  end;
  IB.UnLock;
  VB.UnLock;
end;
//TGroup END

//TBlock BEGIN
procedure TBlock.BuildFrame;
const
  BSIZE = 0.2;
  BHEIGHT = 1;
  function AddVec(const v: TG2Vec3; const n: TG2Vec3; const t: TG2Vec2): Word;
  begin
    with Frame do
    begin
      Result := Length(Vec);
      SetLength(Vec, Length(Vec) + 1);
      SetLength(Nrm, Length(Nrm) + 1);
      SetLength(Tex, Length(Tex) + 1);
      Vec[High(Vec)] := v;
      Nrm[High(Nrm)] := n;
      Tex[High(Tex)] := t;
    end;
  end;
  procedure AddInd(const Index: Word);
  begin
    with Frame do
    begin
      SetLength(Ind, Length(Ind) + 1);
      Ind[High(Ind)] := Index;
    end;
  end;
  procedure AddTri(const i1, i2, i3: Word);
  begin
    with Frame do
    begin
      AddInd(i3); AddInd(i2); AddInd(i1);
    end;
  end;
var
  i, j: Integer;
  Edg: array of record
    v0, v1: TG2Vec3;
    rb0, rb1: TG2Vec3;
    lb0, lb1: TG2Vec3;
    m: TG2Vec3;
    nr, nl: TG2Vec3;
    iv0, iv1: Word;
    irb0, irb1: Word;
    ilb0, ilb1: Word;
    im: Word;
  end;
  n1, n2, nbr, dir: TG2Vec2;
  n: TG2Vec3;
begin
  SetLength(Edg, Length(p));
  for i := 0 to High(p) do
  with Edg[i] do
  begin
    v0 := G2Vec3(p[i].x, 0, p[i].y);
    v1 := G2Vec3(p[i].x, BHEIGHT, p[i].y);
    if i = 0 then nbr := p[High(p)] else nbr := p[i - 1];
    dir := nbr - p[i];
    nl := G2Vec3(dir.x, 0, dir.y);
    nl.Normalize;
    if i = High(p) then nbr := p[0] else nbr := p[i + 1];
    dir := nbr - p[i];
    nr := G2Vec3(dir.x, 0, dir.y);
    nr.Normalize;
  end;
  for i := 0 to High(Edg) do
  with Edg[i] do
  begin
    rb0 := v0 + nr * BSIZE;
    lb0 := v0 + nl * BSIZE;
    rb1 := v1 + nr * BSIZE; rb1.y := rb1.y - BSIZE;
    lb1 := v1 + nl * BSIZE; lb1.y := lb1.y - BSIZE;
    n1 := G2Vec2(nr.z, -nr.x);
    n2 := G2Vec2(-nl.z, nl.x);
    dir := n1 + n2;
    dir.Normalize;
    dir := dir * BSIZE;
    m := v1 + G2Vec3(dir.x, 0, dir.y);
    n1 := -n1; n2 := -n2;
    n := G2Vec3(n1.x + n2.x, 0, n1.y + n2.y); n.Normalize;
    iv0 := AddVec(v0, n, G2Vec2(0, 0));
    n := G2Vec3(n1.x + n2.x, 1, n1.y + n2.y); n.Normalize;
    iv1 := AddVec(v1, n, G2Vec2(0, 0));
    n := G2Vec3(n1.x, 0, n1.y);
    irb0 := AddVec(rb0, n, G2Vec2(0, 1));
    irb1 := AddVec(rb1, n, G2Vec2(0, 1));
    n := G2Vec3(n2.x, 0, n2.y);
    ilb0 := AddVec(lb0, n, G2Vec2(0, 1));
    ilb1 := AddVec(lb1, n, G2Vec2(0, 1));
    im := AddVec(m, G2Vec3(0, 1, 0), G2Vec2(0, 1));
  end;
  for i := 0 to High(Edg) do
  begin
    if i = High(Edg) then j := 0 else j := i + 1;
    with Edg[i] do
    begin
      AddTri(iv0, iv1, irb1); AddTri(iv0, irb1, irb0);
      AddTri(ilb0, ilb1, iv1); AddTri(ilb0, iv1, iv0);
    end;
    AddTri(Edg[i].iv1, Edg[i].im, Edg[j].im);
    AddTri(Edg[i].iv1, Edg[j].im, Edg[j].iv1);
    AddTri(Edg[i].irb1, Edg[i].iv1, Edg[j].iv1);
    AddTri(Edg[i].irb1, Edg[j].iv1, Edg[j].ilb1);
  end;
  {for i := 0 to High(Edg) do
  begin
    if i = High(Edg) then j := 0 else j := i + 1;
    Edg[i].irb0 := AddVec(Edg[i].rb0, G2Vec3(0, 0, 0), G2Vec2(0.75, 0));
    Edg[i].irb1 := AddVec(Edg[i].rb1, G2Vec3(0, 0, 0), G2Vec2(0.75, 0.25));
    Edg[j].ilb0 := AddVec(Edg[j].lb0, G2Vec3(0, 0, 0), G2Vec2(1, 0.25));
    Edg[j].ilb1 := AddVec(Edg[j].lb1, G2Vec3(0, 0, 0), G2Vec2(1, 0));
    if i = 0 then
    Edg[i].im := AddVec(Edg[i].m, G2Vec3(0, 0, 0), G2Vec2(0.5, 0.75))
    else
    Edg[i].im := AddVec(Edg[i].m, G2Vec3(0, 0, 0), G2Vec2(0, 1));
    AddTri(Edg[i].irb0, Edg[i].irb1, Edg[j].ilb1);
    AddTri(Edg[i].irb0, Edg[j].ilb1, Edg[j].ilb0);
  end;
  for i := 1 to High(Edg) - 1 do
  AddTri(Edg[0].im, Edg[i].im, Edg[i + 1].im);}
end;

procedure TBlock.BuildExter;
const
  BSIZE = 0.01;
  BHEIGHT = 1;
  TEXSCALE = 0.2;
  function AddVec(const v: TG2Vec3; const n: TG2Vec3; const t: TG2Vec2): Word;
  begin
    with Exter do
    begin
      Result := Length(Vec);
      SetLength(Vec, Length(Vec) + 1);
      SetLength(Nrm, Length(Nrm) + 1);
      SetLength(Tex, Length(Tex) + 1);
      Vec[High(Vec)] := v;
      Nrm[High(Nrm)] := n;
      Tex[High(Tex)] := t;
    end;
  end;
  procedure AddInd(const Index: Word);
  begin
    with Exter do
    begin
      SetLength(Ind, Length(Ind) + 1);
      Ind[High(Ind)] := Index;
    end;
  end;
  procedure AddTri(const i1, i2, i3: Word);
  begin
    with Exter do
    begin
      AddInd(i3); AddInd(i2); AddInd(i1);
    end;
  end;
var
  i, j: Integer;
  nbrl, nbrr: Integer;
  e: array of record
    v0, v1: TG2Vec3;
    iv0r, iv0l, iv1: Word;
    n, nr, nl: TG2Vec2;
  end;
  tex: TG2Vec2;
begin
  SetLength(e, Length(p));
  for i := 0 to High(e) do
  with e[i] do
  begin
    if i = 0 then nbrl := High(e) else nbrl := i - 1;
    if i = High(e) then nbrr := 0 else nbrr := i + 1;
    nl := (p[i] - p[nbrl]).Perp; nl.Normalize;
    nr := (p[nbrr] - p[i]).Perp; nr.Normalize;
    n := nl + nr; n.Normalize;
    v0 := G2Vec3(p[i].x - n.x * BSIZE, 0, p[i].y - n.y * BSIZE);
    v1 := G2Vec3(p[i].x - n.x * BSIZE, BHEIGHT - BSIZE, p[i].y - n.y * BSIZE);
    tex := (p[i] + nr * BHEIGHT) * TEXSCALE;
    iv0r := AddVec(v0, G2Vec3(nr.x, 0, nr.y), tex);
    tex := (p[i] + nl * BHEIGHT) * TEXSCALE;
    iv0l := AddVec(v0, G2Vec3(nl.x, 0, nl.y), tex);
    tex := p[i] * TEXSCALE;
    iv1 := AddVec(v1, G2Vec3(n.x, 1, n.y), tex);
  end;
  for i := 0 to High(e) do
  begin
    if i = High(e) then j := 0 else j := i + 1;
    AddTri(e[i].iv0r, e[i].iv1, e[j].iv1);
    AddTri(e[i].iv0r, e[j].iv1, e[j].iv0l);
  end;
  for i := 1 to High(e) - 1 do
  AddTri(e[0].iv1, e[i + 1].iv1, e[i].iv1);
end;
//TBlock END

//TBall BEGIN
constructor TBall.Create(const Group: TG2Group);
var
  i, j: Integer;
  b: Boolean;
begin
  inherited Create(Group);
  Fixed := True;
  FixedPos := Random(Trunc(App.Bar.Len * 100)) * 0.005;
  FixedPos := FixedPos - FixedPos * 0.5;
  Inst := App.MshBall.CreateInstance;
  R := 0.016;
  m.SetIdentity;
  Rot.SetIdentity;
  Pos.SetValue(0, 0, 0);
  V := G2Vec2(0, 0);
  Light := 0;
  for i := 2 to 7 do
  begin
    b := True;
    for j := 0 to App.Balls.Count - 1 do
    begin
      if TBall(App.Balls[j]).Light = i then
      begin
        b := False;
        Break;
      end;
    end;
    if b then Light := i;
    if Light >= 2 then Break;
  end;
end;

destructor TBall.Destroy;
begin
  if Light >= 2 then
  begin
    App.Gfx.Lights[Light].Enabled := False;
    App.Gfx.Lights[Light].SetToDevice;
  end;
  Inst.Free;
  inherited Destroy;
end;

procedure TBall.Render;
begin
  Inst.Transform := m;
  Inst.Render;
end;

procedure TBall.Update;
const
  COLR = 0.3;
  ITER = 20;
var
  rv: TG2Vec3;
  t, i: Integer;
  pb: PBlock;
  c: TCSCircle;
  Resp: TCSResponse;
  n: TG2Vec2;
  p: TG2Vec2;
  g, Len: Single;
  mat: TG2Mat;
  TmpV: TG2Vec2;
begin
  if Dead then Exit;
  if Fixed then
  begin
    Pos.x := App.Bar.Pos + FixedPos;
    Pos.y := R;
    Pos.z := 1.6;
    Rot.SetIdentity;
  end
  else
  begin
    Len := V.Len;
    if Abs(V.y) < 0.9 then
    begin
      V.Normalize;
      V.y := V.y - 0.001;
      V.Normalize;
      V := V * Len;
    end;
    TmpV := V / ITER;
    for t := 0 to ITER - 1 do
    begin
      Pos.x := Pos.x + TmpV.x;
      Pos.z := Pos.z + TmpV.y;
      Pos.y := R;
      for i := 0 to App.Blocks.Count - 1 do
      begin
        pb := PBlock(App.Blocks[i]);
        if pb^.Dead then Continue;
        if Proximity(pb) then
        begin
          c.Pos.x := Pos.x;
          c.Pos.y := Pos.z;
          c.Radius := COLR;
          if CSCircleVsPolyIntersect(c, pb^.Poly, @Resp) then
          begin
            n := G2Vec2(Resp.N.x, Resp.N.y);
            p := n * (Resp.D);
            Pos := Pos + G2Vec3(p.x, 0, p.y);
            TmpV := G2Vec2Reflect(TmpV, n);

            App.SndInstBell1.Play;
            if pb^.Level > 0 then
            begin
              if pb^.Level > 1 then
              begin
                Dec(pb^.Level);
                if Assigned(pb^.Group) then
                pb^.Group^.Build;
              end
              else
              begin
                pb^.DieTime := App.AppTime;
                pb^.Dead := True;
                if Assigned(pb^.Group) then
                begin
                  pb^.DieMat := pb^.Group^.Mat3D;
                  pb^.Group^.Blocks.Remove(pb);
                  pb^.Group^.Build;
                  pb^.Group := nil;
                end;
                App.DieBlocks.Add(pb);
              end;
            end;
            
          end;
        end;
      end;
    end;
    V := TmpV * ITER;
    if (Pos.z < 1.6) and (Pos.z > 0.6)
    and (Pos.x > App.Bar.Pos - App.Bar.Len * 0.5)
    and (Pos.x < App.Bar.Pos + App.Bar.Len * 0.5)
    and (V * G2Vec2(0, 1) < 0) then
    begin
      g := (Pos.x - (App.Bar.Pos - App.Bar.Len * 0.5)) / App.Bar.Len;
      g := G2LerpFloat(Pi - 0.3, 0.3, g);
      Len := V.Len;
      mat.SetRotationZ(g);
      V := (G2Vec2(1, 0) * mat) * Len;
      App.SndInstBell3.Play;
    end;
    if Pos.x + R > App.Map.W then
    begin
      Pos.x := App.Map.W - R - 1E-8;
      V.x := -Abs(V.x);
      App.SndInstBell2.Play;
    end;
    if Pos.x - R < 0 then
    begin
      Pos.x := R + 1E-8;
      V.x := Abs(V.x);
      App.SndInstBell2.Play;
    end;
    if Pos.z + r > App.Map.H then
    begin
      Pos.z := App.Map.H - R - 1E-8;
      V.y := -Abs(V.y);
      App.SndInstBell2.Play;
    end;
    if Pos.z - r < 0 then
    begin
      Die;
      if App.Lives > 0 then
      begin
        Dec(App.Lives);
        App.AddBall(0.05);
      end;
      App.SndBallLost.Play;
    end;
    rv := G2Vec3(-V.y, 0, V.x);
    Rot.Rotate(rv, V.Len * 4 / R);
  end;
  if Light >= 2 then
  begin
    App.Gfx.Lights[Light].SetPointLight(
      Pos.x, 1, Pos.z, $ff888888, 5
    );
    App.Gfx.Lights[Light].Enabled := True;
    App.Gfx.Lights[Light].SetToDevice;
  end;
  m.SetRotation(Rot);
  m.Translate(Pos);
end;

function TBall.Proximity(const Block: PBlock): Boolean;
const
  COLR = 0.4;
begin
  with Block^.Bonds do
  Result := (
    (Pos.x > Left - COLR)
    and (Pos.z > Top - COLR)
    and (Pos.x < Right + COLR)
    and (Pos.z < Bottom + COLR)
  );
end;
//TBall END

var
  msg: TMsg;
begin
  App := TMyApp.Create;       
  App.Initialize;

  FillChar(msg, SizeOf(msg), 0);
  while App.Running
  and (msg.message <> WM_QUIT)
  and (msg.message <> WM_DESTROY)
  and (msg.message <> WM_CLOSE) do
  begin
    if PeekMessage(msg, 0, 0, 0, PM_REMOVE) then
    begin
      TranslateMessage(msg);
      DispatchMessage(msg);
    end
    else
    if App.Tmr.Mode = tmManual then
    App.Tmr.OnTimer;
  end;

  App.Free;

  ExitCode := 0;
end.

