//G2Math v1.0
//Author: Dan
unit G2Math;

{$include Gen2.inc}

interface

uses
  Types,
  SysUtils,
  Math,
  Direct3D9,
  D3DX9;

type
  UInt8 = Byte;
  Int8 = ShortInt;
  UInt16 = Word;
  Int16 = SmallInt;
  UInt32 = DWord;
  Int32 = Integer;
  
  PG2MatRef = ^TG2MatRef;
  TG2MatRef = TD3DXMatrix;

  PG2Mat2Ref = ^TG2Mat2Ref;
  TG2Mat2Ref = array[0..1, 0..1] of Single;

  PG2Vec2Ref = ^TG2Vec2Ref;
  TG2Vec2Ref = TD3DXVector2;

  PG2Vec3Ref = ^TG2Vec3Ref;
  TG2Vec3Ref = TD3DXVector3;

  PG2Vec4Ref = ^TG2Vec4Ref;
  TG2Vec4Ref = TD3DXVector4;

  PG2QuatRef = ^TG2QuatRef;
  TG2QuatRef = TD3DXQuaternion;

  PG2PlaneRef = ^TG2PlaneRef;
  TG2PlaneRef = TD3DXPlane;

  PG2Mat = ^TG2Mat;
  TG2Mat = packed record
  public
    e00, e01, e02, e03: Single;
    e10, e11, e12, e13: Single;
    e20, e21, e22, e23: Single;
    e30, e31, e32, e33: Single;
    class operator Explicit(const m: TG2Mat): TG2MatRef;
    class operator Explicit(const m: TG2MatRef): TG2Mat;
    class operator Implicit(const m: TG2Mat): TG2MatRef;
    class operator Implicit(const m: TG2MatRef): TG2Mat;
    class operator Equal(const m1, m2: TG2Mat): Boolean;
    class operator NotEqual(const m1, m2: TG2Mat): Boolean;
    class operator Add(const m1, m2: TG2Mat): TG2Mat;
    class operator Subtract(const m1, m2: TG2Mat): TG2Mat;
    class operator Multiply(const m1, m2: TG2Mat): TG2Mat;
    procedure SetIdentity; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetScaling(const _X, _Y, _Z: Single); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetScaling(const _v: TG2Vec3Ref); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetTranslation(const _X, _Y, _Z: Single); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetTranslation(const _v: TG2Vec3Ref); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetRotationX(const _Angle: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetRotationY(const _Angle: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetRotationZ(const _Angle: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetRotation(const _X, _Y, _Z, _Angle: Single); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetRotation(const _q: TG2QuatRef); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Scale(const X, Y, Z: Single); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Scale(const v: TG2Vec3Ref); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Translate(const X, Y, Z: Single); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Translate(const v: TG2Vec3Ref); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure RotateX(const Angle: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure RotateY(const Angle: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure RotateZ(const Angle: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Rotate(const X, Y, Z, Angle: Single); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Rotate(const q: TG2QuatRef); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Decompose(const Scaling: PG2Vec3Ref; const Rotation: PG2QuatRef; const Translation: PG2Vec3Ref); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Determinant: Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Transpose: TG2Mat; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Inverse: TG2Mat;
  end;

  PG2Mat2 = ^TG2Mat2;
  TG2Mat2 = packed record
  public
    e00, e01: Single;
    e10, e11: Single;
    class operator Explicit(const m: TG2Mat2): TG2Mat2Ref;
    class operator Explicit(const m: TG2Mat2Ref): TG2Mat2;
    class operator Implicit(const m: TG2Mat2): TG2Mat2Ref;
    class operator Implicit(const m: TG2Mat2Ref): TG2Mat2;
    class operator Equal(const m1, m2: TG2Mat2): Boolean;
    class operator NotEqual(const m1, m2: TG2Mat2): Boolean;
    class operator Add(const m1, m2: TG2Mat2): TG2Mat2;
    class operator Subtract(const m1, m2: TG2Mat2): TG2Mat2;
    class operator Multiply(const m1, m2: TG2Mat2): TG2Mat2;
    class operator Multiply(const m: TG2Mat2; s: Single): TG2Mat2;
    procedure SetIdentity; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetScaling(const _X, _Y: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetRotation(const _Angle: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Scale(const X, Y: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Rotate(const Angle: Single);
    function Determinant: Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Transpose: TG2Mat2;
    function Inverse: TG2Mat2;
    function GetRotation: Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
  end;

  PG2Vec2 = ^TG2Vec2;
  TG2Vec2 = record
  public
    x, y: Single;
    class operator Negative(const v: TG2Vec2): TG2Vec2;
    class operator Explicit(const v: TG2Vec2): TG2Vec2Ref;
    class operator Explicit(const v: TG2Vec2Ref): TG2Vec2;
    class operator Explicit(const v: TPoint): TG2Vec2;
    class operator Implicit(const v: TG2Vec2): TG2Vec2Ref;
    class operator Implicit(const v: TG2Vec2Ref): TG2Vec2;
    class operator Implicit(const v: TPoint): TG2Vec2;
    class operator Equal(const v1, v2: TG2Vec2): Boolean;
    class operator NotEqual(const v1, v2: TG2Vec2): Boolean;
    class operator Add(const v1, v2: TG2Vec2): TG2Vec2;
    class operator Add(const v: TG2Vec2; const s: Single): TG2Vec2;
    class operator Subtract(const v1, v2: TG2Vec2): TG2Vec2;
    class operator Subtract(const v: TG2Vec2; const s: Single): TG2Vec2;
    class operator Multiply(const v1, v2: TG2Vec2): Single;
    class operator Multiply(const v: TG2Vec2; const s: Single): TG2Vec2;
    class operator Multiply(const v: TG2Vec2; const m: TG2Mat): TG2Vec2;
    class operator Multiply(const v: TG2Vec2; const m: TG2Mat2): TG2Vec2;
    class operator Divide(const v: TG2Vec2; const s: Single): TG2Vec2;
    procedure SetValue(const _X, _Y: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Normalize; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Len: Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Dot(const v: TG2Vec2): Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Cross(const v: TG2Vec2): Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Angle(const v: TG2Vec2): Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Perp: TG2Vec2;
  end;

  PG2Vec3 = ^TG2Vec3;
  TG2Vec3 = packed record
  public
    x, y, z: Single;
    class operator Negative(const v: TG2Vec3): TG2Vec3; 
    class operator Explicit(const v: TG2Vec3): TG2Vec3Ref; 
    class operator Explicit(const v: TG2Vec3Ref): TG2Vec3; 
    class operator Implicit(const v: TG2Vec3): TG2Vec3Ref; 
    class operator Implicit(const v: TG2Vec3Ref): TG2Vec3; 
    class operator Equal(const v1, v2: TG2Vec3): Boolean; 
    class operator NotEqual(const v1, v2: TG2Vec3): Boolean; 
    class operator Add(const v1, v2: TG2Vec3): TG2Vec3;
    class operator Add(const v: TG2Vec3; const s: Single): TG2Vec3; 
    class operator Subtract(const v1, v2: TG2Vec3): TG2Vec3;
    class operator Subtract(const v: TG2Vec3; const s: Single): TG2Vec3; 
    class operator Multiply(const v1, v2: TG2Vec3): Single; 
    class operator Multiply(const v: TG2Vec3; const s: Single): TG2Vec3; 
    class operator Multiply(const v: TG2Vec3; const m: TG2Mat): TG2Vec3; 
    class operator Divide(const v: TG2Vec3; const s: Single): TG2Vec3;
    procedure SetValue(const _X, _Y, _Z: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Normalize; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure TransformCoord(const m: TG2Mat); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure TransformNormal(const m: TG2Mat); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Len: Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Dot(const v: TG2Vec3): Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Cross(const v: TG2Vec3): TG2Vec3; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Angle(const v: TG2Vec3): Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF} 
  end;

  PG2Vec4 = ^TG2Vec4;
  TG2Vec4 = record
  public
    x, y, z, w: Single;
    class operator Negative(const v: TG2Vec4): TG2Vec4;
    class operator Explicit(const v: TG2Vec4): TG2Vec4Ref; 
    class operator Explicit(const v: TG2Vec4Ref): TG2Vec4; 
    class operator Implicit(const v: TG2Vec4): TG2Vec4Ref; 
    class operator Implicit(const v: TG2Vec4Ref): TG2Vec4; 
    class operator Equal(const v1, v2: TG2Vec4): Boolean; 
    class operator NotEqual(const v1, v2: TG2Vec4): Boolean; 
    class operator Add(const v1, v2: TG2Vec4): TG2Vec4;
    class operator Add(const v: TG2Vec4; const s: Single): TG2Vec4; 
    class operator Subtract(const v1, v2: TG2Vec4): TG2Vec4;
    class operator Subtract(const v: TG2Vec4; const s: Single): TG2Vec4; 
    class operator Multiply(const v1, v2: TG2Vec4): Single; 
    class operator Multiply(const v: TG2Vec4; const s: Single): TG2Vec4;
    class operator Multiply(const v: TG2Vec4; const m: TG2Mat): TG2Vec4; 
    class operator Divide(const v: TG2Vec4; const s: Single): TG2Vec4;
    procedure SetValue(const _X, _Y, _Z, _W: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Normalize; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Len: Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Dot(const v: TG2Vec4): Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Cross(const v1, v2: TG2Vec4): TG2Vec3; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Angle(const v: TG2Vec4): Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF} 
  end;

  PG2Quat = ^TG2Quat;
  TG2Quat = record
  public
    x, y, z, w: Single;
    class operator Explicit(const q: TG2Quat): TG2QuatRef;
    class operator Explicit(const q: TG2QuatRef): TG2Quat;
    class operator Implicit(const q: TG2Quat): TG2QuatRef;
    class operator Implicit(const q: TG2QuatRef): TG2Quat;
    class operator Equal(const q1, q2: TG2Quat): Boolean;
    class operator NotEqual(const q1, q2: TG2Quat): Boolean;
    class operator Multiply(const q1, q2: TG2Quat): TG2Quat;
    procedure SetValue(const _X, _Y, _Z, _W: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF} 
    procedure SetIdentity; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetRotation(const Axis: TG2Vec3; const Angle: Single); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetRotation(const m: TG2Mat); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Rotate(const Axis: TG2Vec3; const Angle: Single); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Rotate(const m: TG2Mat); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Normalize; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure GetRotation(var Axis: TG2Vec3; var Angle: Single); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure GetRotation(var m: TG2Mat); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Conjugate: TG2Quat; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function Inverse: TG2Quat; {$IFDEF G2_USE_INLINE} inline; {$ENDIF} 
  end;

  PG2AABox = ^TG2AABox;
  TG2AABox = record
  public
    MinV, MaxV: TG2Vec3;
    class operator Equal(const b1, b2: TG2AABox): Boolean;
    class operator NotEqual(const b1, b2: TG2AABox): Boolean;
    procedure SetValue(const _MinV, _MaxV: TG2Vec3); {$IFDEF G2_USE_INLINE} inline; {$ENDIF} 
  end;

  PG2Box = ^TG2Box;
  TG2Box = record
  public
    C, vx, vy, vz: TG2Vec3;
    class operator Equal(const b1, b2: TG2Box): Boolean;
    class operator NotEqual(const b1, b2: TG2Box): Boolean;
    class operator Multiply(const b: TG2Box; const m: TG2Mat): TG2Box;
    procedure SetValue(const _C, _vx, _vy, _vz: TG2Vec3); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function AABox: TG2AABox; 
  end;

  PG2Sphere = ^TG2Sphere;
  TG2Sphere = record
  public
    C: TG2Vec3;
    R: Single;
    class operator Equal(const s1, s2: TG2Sphere): Boolean;
    class operator NotEqual(const s1, s2: TG2Sphere): Boolean;
    procedure SetValue(const _C: TG2Vec3; const _R: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
  end;

  PG2Plane = ^TG2Plane;
  TG2Plane = record
  public
    N: TG2Vec3;
    D: Single;
    class operator Explicit(const p: TG2Plane): TG2PlaneRef;
    class operator Explicit(const p: TG2PlaneRef): TG2Plane;
    class operator Implicit(const p: TG2Plane): TG2PlaneRef;
    class operator Implicit(const p: TG2PlaneRef): TG2Plane;
    class operator Equal(const p1, p2: TG2Plane): Boolean;
    class operator NotEqual(const p1, p2: TG2Plane): Boolean;
    class operator Multiply(const p: TG2Plane; const m: TG2Mat): TG2Plane; 
    procedure SetValue(const _A, _B, _C, _D: Single); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetPlane(const _V, _N: TG2Vec3); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure SetPlane(const _v1, _v2, _v3: TG2Vec3); overload; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure Normalize; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function DistanceToPoint(const v: TG2Vec3): Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function IntersectTri(const v1, v2, v3: TG2Vec3): Boolean; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function IntersectAABox(const b: TG2AABox): Boolean; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function IntersectSphere(const s: TG2Sphere): Boolean; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
  end;
  
  PG2Ray = ^TG2Ray;
  TG2Ray = record
  public
    Origin: TG2Vec3;
    Dir: TG2Vec3;
    class operator Equal(const p1, p2: TG2Plane): Boolean;
    class operator NotEqual(const p1, p2: TG2Plane): Boolean;
    class operator Multiply(const r: TG2Ray; m: TG2Mat): TG2Ray;
    procedure SetValue(const _Origin, _Dir: TG2Vec3);
    procedure Normalize;
    procedure Transform(const m: TG2Mat); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    procedure TransformInverse(const m: TG2Mat); {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function PointAtDistance(const Distance: Single): TG2Vec3; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function IntersectTri(const v1, v2, v3: TG2Vec3; var U, V, Distance: Single): Boolean; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function IntersectSphere(const s: TG2Sphere): Boolean; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function IntersectAABox(const b: TG2AABox): Boolean; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function IntersectPlane(const Plane: TG2Plane; const BackFace: Boolean; var Distance: Single): Boolean; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
  end;

  PG2Vec2Array = ^TG2Vec2Array;
  TG2Vec2Array = array[0..0] of TG2Vec2;

  PG2Vec3Array = ^TG2Vec3Array;
  TG2Vec3Array = array[0..0] of TG2Vec3;

  PG2Vec4Array = ^TG2Vec4Array;
  TG2Vec4Array = array[0..0] of TG2Vec4;

  TG2ProcMatAdd = procedure(const OutM, m1, m2: PG2Mat);
  TG2ProcMatSub = procedure(const OutM, m1, m2: PG2Mat);
  TG2ProcMat2Mul = procedure(const OutM, m1, m2: PG2Mat2);
  TG2ProcVec2Mat2Mul = procedure(const OutV, v: PG2Vec2; const m: PG2Mat2);

var
  G2MatAdd: TG2ProcMatAdd;
  G2MatSub: TG2ProcMatSub;
  G2Mat2Mul: TG2ProcMat2Mul;
  G2Vec2Mat2Mul: TG2ProcVec2Mat2Mul;

function G2Vec2(const _X, _Y: Single): TG2Vec2; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
function G2Vec3(const _X, _Y, _Z: Single): TG2Vec3; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
function G2Vec4(const _X, _Y, _Z, _W: Single): TG2Vec4; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
function G2Quat(const _X, _Y, _Z, _W: Single): TG2Quat; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
function G2LerpFloat(const f1, f2, s: Single): Single; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
function G2LerpVec2(const v1, v2: TG2Vec2; const s: Single): TG2Vec2; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
function G2LerpVec3(const v1, v2: TG2Vec3; const s: Single): TG2Vec3; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
function G2LerpVec4(const v1, v2: TG2Vec4; const s: Single): TG2Vec4; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
function G2SLerpQuat(const q1, q2: TG2Quat; const s: Single): TG2Quat; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
function G2Vec2InPoly(const v: TG2Vec2; const VArr: PG2Vec2Array; const VCount: Integer): Boolean; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
function G2Vec2Reflect(const v, n: TG2Vec2): TG2Vec2; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}

procedure G2MatAddSSE(const OutM, m1, m2: PG2Mat);
procedure G2MatAddStandard(const OutM, m1, m2: PG2Mat);
procedure G2MatSubSSE(const OutM, m1, m2: PG2Mat);
procedure G2MatSubStandard(const OutM, m1, m2: PG2Mat);
procedure G2Mat2MulSSE(const OutM, m1, m2: PG2Mat2);
procedure G2Mat2MulStandard(const OutM, m1, m2: PG2Mat2);
procedure G2Vec2Mat2MulSSE(const OutV, v: PG2Vec2; const m: PG2Mat2);
procedure G2Vec2Mat2MulStandard(const OutV, v: PG2Vec2; const m: PG2Mat2);

const
  TwoPi = Pi * 2;
  HalfPi = Pi * 0.5;
  QuatPi = Pi * 0.25;
  Rcp255 = 1 / 255;

implementation

var
  SysMMX: Boolean = False;
  SysSSE: Boolean = False;
  SysSSE2: Boolean = False;

//TG2Mat BEGIN
class operator TG2Mat.Explicit(const m: TG2Mat): TG2MatRef;
begin
  Result := PG2MatRef(@m)^;
end;

class operator TG2Mat.Explicit(const m: TG2MatRef): TG2Mat;       
begin
  Result := PG2Mat(@m)^;
end;

class operator TG2Mat.Implicit(const m: TG2Mat): TG2MatRef;   
begin
  Result := PG2MatRef(@m)^;
end;

class operator TG2Mat.Implicit(const m: TG2MatRef): TG2Mat; 
begin
  Result := PG2Mat(@m)^;
end;

class operator TG2Mat.Equal(const m1, m2: TG2Mat): Boolean;   
begin
  Result := CompareMem(@m1, @m2, SizeOf(TG2Mat));
end;

class operator TG2Mat.NotEqual(const m1, m2: TG2Mat): Boolean;  
begin
  Result := not CompareMem(@m1, @m2, SizeOf(TG2Mat));
end;

class operator TG2Mat.Add(const m1, m2: TG2Mat): TG2Mat;     
begin
  G2MatAdd(@Result, @m1, @m2);
end;

class operator TG2Mat.Subtract(const m1, m2: TG2Mat): TG2Mat;  
begin
  G2MatSub(@Result, @m1, @m2);
end;

class operator TG2Mat.Multiply(const m1, m2: TG2Mat): TG2Mat;       
begin
  D3DXMatrixMultiply(PG2MatRef(@Result)^, PG2MatRef(@m1)^, PG2MatRef(@m2)^);
end;

procedure TG2Mat.SetIdentity;
begin
  FillChar(Self, SizeOf(TG2Mat), 0);
  e00 := 1; e11 := 1; e22 := 1; e33 := 1;
end;

procedure TG2Mat.SetScaling(const _X, _Y, _Z: Single);
begin
  D3DXMatrixScaling(PG2MatRef(@Self)^, _X, _Y, _Z);
end;

procedure TG2Mat.SetScaling(const _v: TG2Vec3Ref);   
begin
  D3DXMatrixScaling(PG2MatRef(@Self)^, _v.x, _v.y, _v.z);
end;

procedure TG2Mat.SetTranslation(const _X, _Y, _Z: Single);
begin
  D3DXMatrixTranslation(PG2MatRef(@Self)^, _X, _Y, _Z);
end;

procedure TG2Mat.SetTranslation(const _v: TG2Vec3Ref);
begin
  D3DXMatrixTranslation(PG2MatRef(@Self)^, _v.x, _v.y, _v.z);
end;

procedure TG2Mat.SetRotationX(const _Angle: Single);  
begin
  D3DXMatrixRotationX(PG2MatRef(@Self)^, _Angle);
end;

procedure TG2Mat.SetRotationY(const _Angle: Single);  
begin
  D3DXMatrixRotationY(PG2MatRef(@Self)^, _Angle);
end;

procedure TG2Mat.SetRotationZ(const _Angle: Single);  
begin
  D3DXMatrixRotationZ(PG2MatRef(@Self)^, _Angle);
end;

procedure TG2Mat.SetRotation(const _X, _Y, _Z, _Angle: Single);
begin
  D3DXMatrixRotationAxis(PG2MatRef(@Self)^, D3DXVector3(_X, _Y, _Z), _Angle);
end;

procedure TG2Mat.SetRotation(const _q: TG2QuatRef);
begin
  D3DXMatrixRotationQuaternion(PG2MatRef(@Self)^, _q);
end;

procedure TG2Mat.Scale(const X, Y, Z: Single);
var
  m: TG2Mat;
begin
  m.SetScaling(X, Y, Z);
  D3DXMatrixMultiply(PG2MatRef(@Self)^, PG2MatRef(@Self)^, PG2MatRef(@m)^);
end;

procedure TG2Mat.Scale(const v: TG2Vec3Ref);
var
  m: TG2Mat;
begin
  m.SetScaling(v);
  D3DXMatrixMultiply(PG2MatRef(@Self)^, PG2MatRef(@Self)^, PG2MatRef(@m)^);
end;

procedure TG2Mat.Translate(const X, Y, Z: Single);
var
  m: TG2Mat;
begin
  m.SetTranslation(X, Y, Z);
  D3DXMatrixMultiply(PG2MatRef(@Self)^, PG2MatRef(@Self)^, PG2MatRef(@m)^);
end;

procedure TG2Mat.Translate(const v: TG2Vec3Ref);
var
  m: TG2Mat;
begin
  m.SetTranslation(v);
  D3DXMatrixMultiply(PG2MatRef(@Self)^, PG2MatRef(@Self)^, PG2MatRef(@m)^);
end;

procedure TG2Mat.RotateX(const Angle: Single);
var
  m: TG2Mat;
begin
  m.SetRotationX(Angle);
  D3DXMatrixMultiply(PG2MatRef(@Self)^, PG2MatRef(@Self)^, PG2MatRef(@m)^);
end;

procedure TG2Mat.RotateY(const Angle: Single);
var
  m: TG2Mat;
begin
  m.SetRotationY(Angle);
  D3DXMatrixMultiply(PG2MatRef(@Self)^, PG2MatRef(@Self)^, PG2MatRef(@m)^);
end;

procedure TG2Mat.RotateZ(const Angle: Single);
var
  m: TG2Mat;
begin
  m.SetRotationZ(Angle);
  D3DXMatrixMultiply(PG2MatRef(@Self)^, PG2MatRef(@Self)^, PG2MatRef(@m)^);
end;

procedure TG2Mat.Rotate(const X, Y, Z, Angle: Single);
var
  m: TG2Mat;
begin
  m.SetRotation(X, Y, Z, Angle);
  D3DXMatrixMultiply(PG2MatRef(@Self)^, PG2MatRef(@Self)^, PG2MatRef(@m)^);
end;

procedure TG2Mat.Rotate(const q: TG2QuatRef);
var
  m: TG2Mat;
begin
  m.SetRotation(q);
  D3DXMatrixMultiply(PG2MatRef(@Self)^, PG2MatRef(@Self)^, PG2MatRef(@m)^);
end;

procedure TG2Mat.Decompose(const Scaling: PG2Vec3Ref; const Rotation: PG2QuatRef; const Translation: PG2Vec3Ref);
begin
  D3DXMatrixDecompose(
    PD3DXVector3(Scaling),
    PD3DXQuaternion(Rotation),
    PD3DXVector3(Translation),
    PG2MatRef(@Self)^
  );
end;

function TG2Mat.Determinant: Single;
begin
  Result := D3DXMatrixDeterminant(PG2MatRef(@Self)^);
end;

function TG2Mat.Transpose: TG2Mat;
begin
  D3DXMatrixTranspose(PG2MatRef(@Result)^, PG2MatRef(@Self)^);
end;

function TG2Mat.Inverse: TG2Mat;
begin
  D3DXMatrixInverse(PG2MatRef(@Result)^, nil, PG2MatRef(@Self)^);
end;
//TG2Mat END

//TG2Mat2 BEGIN
class operator TG2Mat2.Explicit(const m: TG2Mat2): TG2Mat2Ref;
begin
  Result := PG2Mat2Ref(@m)^;
end;

class operator TG2Mat2.Explicit(const m: TG2Mat2Ref): TG2Mat2;
begin
  Result := PG2Mat2(@m)^;
end;

class operator TG2Mat2.Implicit(const m: TG2Mat2): TG2Mat2Ref;
begin
  Result := PG2Mat2Ref(@m)^;
end;

class operator TG2Mat2.Implicit(const m: TG2Mat2Ref): TG2Mat2;
begin
  Result := PG2Mat2(@m)^;
end;

class operator TG2Mat2.Equal(const m1, m2: TG2Mat2): Boolean;
begin
  Result := CompareMem(@m1, @m2, SizeOf(TG2Mat2));
end;

class operator TG2Mat2.NotEqual(const m1, m2: TG2Mat2): Boolean;  
begin
  Result := not CompareMem(@m1, @m2, SizeOf(TG2Mat2));
end;

class operator TG2Mat2.Add(const m1, m2: TG2Mat2): TG2Mat2;
begin
  with Result do
  begin
    e00 := m1.e00 + m2.e00; e10 := m1.e10 + m2.e10;
    e01 := m1.e01 + m2.e01; e11 := m1.e11 + m2.e11;
  end;
end;

class operator TG2Mat2.Subtract(const m1, m2: TG2Mat2): TG2Mat2;   
begin
  with Result do
  begin
    e00 := m1.e00 - m2.e00; e10 := m1.e10 - m2.e10;
    e01 := m1.e01 - m2.e01; e11 := m1.e11 - m2.e11;
  end;
end;

class operator TG2Mat2.Multiply(const m1, m2: TG2Mat2): TG2Mat2;    
begin
  G2Mat2Mul(@Result, @m1, @m2);
end;

class operator TG2Mat2.Multiply(const m: TG2Mat2; s: Single): TG2Mat2; 
begin
  Result.e00 := m.e00 * s;
  Result.e01 := m.e01 * s;
  Result.e10 := m.e10 * s;
  Result.e11 := m.e11 * s;
end;

procedure TG2Mat2.SetIdentity;
begin
  e00 := 1; e10 := 0; e01 := 0; e11 := 1;
end;

procedure TG2Mat2.SetScaling(const _X, _Y: Single); 
begin
  e00 := _X; e10 := 0; e01 := 0; e11 := _Y;
end;

procedure TG2Mat2.SetRotation(const _Angle: Single);
var
  s, c: Extended;
begin
  SinCos(_Angle, s, c);
  e00 := c;
  e10 := -s;
  e01 := s;
  e11 := c;
end;

procedure TG2Mat2.Scale(const X, Y: Single);
var
  m: TG2Mat2;  
begin
  m.SetScaling(X, Y);
  Self := Self * m;
end;

procedure TG2Mat2.Rotate(const Angle: Single); 
var
  m: TG2Mat2;  
begin
  m.SetRotation(Angle);
  Self := Self * m;
end;

function TG2Mat2.Determinant: Single;     
begin
  Result := e00 * e11 - e10 * e01;
end;

function TG2Mat2.Transpose: TG2Mat2;  
var
  rm: TG2Mat2;
begin
  rm.e10 := e01;
  rm.e01 := e10;
  Result := rm;
end;

function TG2Mat2.Inverse: TG2Mat2; 
var
  D: Single;
  rm: TG2Mat2;
begin
  D := 1 / Determinant;
  rm.e00 := e11 * D;
  rm.e01 := -e01 * D;
  rm.e10 := -e10 * D;
  rm.e11 := e00 * D;
  Result := rm;
end;

function TG2Mat2.GetRotation: Single;  
begin
  Result := ArcTan2(e01, e00);
end;
//TG2Mat2 END

//TG2Vec2 BEGIN
class operator TG2Vec2.Negative(const v: TG2Vec2): TG2Vec2;
begin
  Result.x := -v.x;
  Result.y := -v.y;
end;

class operator TG2Vec2.Explicit(const v: TG2Vec2): TG2Vec2Ref;  
begin
  Result := PG2Vec2Ref(@v)^;
end;

class operator TG2Vec2.Explicit(const v: TG2Vec2Ref): TG2Vec2;  
begin
  Result := PG2Vec2(@v)^;
end;

class operator TG2Vec2.Explicit(const v: TPoint): TG2Vec2;      
begin
  Result.x := v.X;
  Result.y := v.Y;
end;

class operator TG2Vec2.Implicit(const v: TG2Vec2): TG2Vec2Ref; 
begin
  Result := PG2Vec2Ref(@v)^;
end;

class operator TG2Vec2.Implicit(const v: TG2Vec2Ref): TG2Vec2;  
begin
  Result := PG2Vec2(@v)^;
end;

class operator TG2Vec2.Implicit(const v: TPoint): TG2Vec2;        
begin
  Result.x := v.X;
  Result.y := v.Y;
end;

class operator TG2Vec2.Equal(const v1, v2: TG2Vec2): Boolean;    
begin
  Result := CompareMem(@v1, @v2, SizeOf(TG2Vec2));
end;

class operator TG2Vec2.NotEqual(const v1, v2: TG2Vec2): Boolean;   
begin
  Result := not CompareMem(@v1, @v2, SizeOf(TG2Vec2));
end;

class operator TG2Vec2.Add(const v1, v2: TG2Vec2): TG2Vec2;       
begin
  Result.x := v1.x + v2.x;
  Result.y := v1.y + v2.y;
end;

class operator TG2Vec2.Add(const v: TG2Vec2; const s: Single): TG2Vec2;
begin
  Result.x := v.x + s;
  Result.y := v.y + s;
end;

class operator TG2Vec2.Subtract(const v1, v2: TG2Vec2): TG2Vec2;      
begin
  Result.x := v1.x - v2.x;
  Result.y := v1.y - v2.y;
end;

class operator TG2Vec2.Subtract(const v: TG2Vec2; const s: Single): TG2Vec2;   
begin
  Result.x := v.x - s;
  Result.y := v.y - s;
end;

class operator TG2Vec2.Multiply(const v1, v2: TG2Vec2): Single;  
begin
  Result := D3DXVec2Dot(PG2Vec2Ref(@v1)^, PG2Vec2Ref(@v2)^);
end;

class operator TG2Vec2.Multiply(const v: TG2Vec2; const s: Single): TG2Vec2;     
begin
  D3DXVec2Scale(PG2Vec2Ref(@Result)^, PG2Vec2Ref(@v)^, s);
end;

class operator TG2Vec2.Multiply(const v: TG2Vec2; const m: TG2Mat): TG2Vec2;  
begin
  D3DXVec2TransformCoord(PG2Vec2Ref(@Result)^, PG2Vec2Ref(@v)^, PG2MatRef(@m)^);
end;

class operator TG2Vec2.Multiply(const v: TG2Vec2; const m: TG2Mat2): TG2Vec2;  
begin
  G2Vec2Mat2Mul(@Result, @v, @m);
end;

class operator TG2Vec2.Divide(const v: TG2Vec2; const s: Single): TG2Vec2;  
var
  RcpS: Single;
begin
  RcpS := 1 / s;
  Result.x := v.x * RcpS;
  Result.y := v.y * RcpS;
end;

procedure TG2Vec2.SetValue(const _X, _Y: Single);
begin
  x := _X; y := _Y;
end;

procedure TG2Vec2.Normalize;
begin
  D3DXVec2Normalize(PG2Vec2Ref(@Self)^, PG2Vec2Ref(@Self)^);
end;

function TG2Vec2.Len: Single;
begin
  Result := D3DXVec2Length(PG2Vec2Ref(@Self)^);
end;

function TG2Vec2.Dot(const v: TG2Vec2): Single;
begin
  Result := D3DXVec2Dot(PG2Vec2Ref(@Self)^, PG2Vec2Ref(@v)^);
end;

function TG2Vec2.Cross(const v: TG2Vec2): Single;
begin
  Result := x * v.y - y * v.x;
end;

function TG2Vec2.Angle(const v: TG2Vec2): Single;
begin
  Result := ArcCos((Self * v) / (Len * v.Len));
end;

function TG2Vec2.Perp: TG2Vec2;
begin
  Result := G2Vec2(-y, x);
end;
//TG2Vec2 END

//TG2Vec3 BEGIN
class operator TG2Vec3.Negative(const v: TG2Vec3): TG2Vec3;
begin
  Result.x := -v.x;
  Result.y := -v.y;
  Result.z := -v.z;
end;

class operator TG2Vec3.Explicit(const v: TG2Vec3): TG2Vec3Ref;  
begin
  Result := PG2Vec3Ref(@v)^;
end;

class operator TG2Vec3.Explicit(const v: TG2Vec3Ref): TG2Vec3;  
begin
  Result := PG2Vec3(@v)^;
end;

class operator TG2Vec3.Implicit(const v: TG2Vec3): TG2Vec3Ref;  
begin
  Result := PG2Vec3Ref(@v)^;
end;

class operator TG2Vec3.Implicit(const v: TG2Vec3Ref): TG2Vec3;   
begin
  Result := PG2Vec3(@v)^;
end;

class operator TG2Vec3.Equal(const v1, v2: TG2Vec3): Boolean;     
begin
  Result := CompareMem(@v1, @v2, SizeOf(TG2Vec3));
end;

class operator TG2Vec3.NotEqual(const v1, v2: TG2Vec3): Boolean;    
begin
  Result := not CompareMem(@v1, @v2, SizeOf(TG2Vec3));
end;

class operator TG2Vec3.Add(const v1, v2: TG2Vec3): TG2Vec3;          
begin
  Result.x := v1.x + v2.x;
  Result.y := v1.y + v2.y;
  Result.z := v1.z + v2.z;
end;

class operator TG2Vec3.Add(const v: TG2Vec3; const s: Single): TG2Vec3;  
begin
  Result.x := v.x + s;
  Result.y := v.y + s;
  Result.z := v.z + s;
end;

class operator TG2Vec3.Subtract(const v1, v2: TG2Vec3): TG2Vec3;     
begin
  Result.x := v1.x - v2.x;
  Result.y := v1.y - v2.y;
  Result.z := v1.z - v2.z;
end;

class operator TG2Vec3.Subtract(const v: TG2Vec3; const s: Single): TG2Vec3; 
begin
  Result.x := v.x - s;
  Result.y := v.y - s;
  Result.z := v.z - s;
end;

class operator TG2Vec3.Multiply(const v1, v2: TG2Vec3): Single;               
begin
  Result := D3DXVec3Dot(PG2Vec3Ref(@v1)^, PG2Vec3Ref(@v2)^);
end;

class operator TG2Vec3.Multiply(const v: TG2Vec3; const s: Single): TG2Vec3;   
begin
  Result.x := v.x * s;
  Result.y := v.y * s;
  Result.z := v.z * s;
end;

class operator TG2Vec3.Multiply(const v: TG2Vec3; const m: TG2Mat): TG2Vec3;  
begin
  D3DXVec3TransformCoord(PG2Vec3Ref(@Result)^, PG2Vec3Ref(@v)^, PG2MatRef(@m)^);
end;

class operator TG2Vec3.Divide(const v: TG2Vec3; const s: Single): TG2Vec3;  
var
  RcpS: Single;
begin
  RcpS := 1 / s;
  Result.x := v.x * RcpS;
  Result.y := v.y * RcpS;
  Result.z := v.z * RcpS;
end;

procedure TG2Vec3.SetValue(const _X, _Y, _Z: Single);
begin
  x := _X; y := _Y; z := _Z;
end;

procedure TG2Vec3.Normalize;
begin
  D3DXVec3Normalize(PG2Vec3Ref(@Self)^, PG2Vec3Ref(@Self)^);
end;

procedure TG2Vec3.TransformCoord(const m: TG2Mat);
begin
  D3DXVec3TransformCoord(PG2Vec3Ref(@Self)^, PG2Vec3Ref(@Self)^, PG2MatRef(@m)^);
end;

procedure TG2Vec3.TransformNormal(const m: TG2Mat);
begin
  D3DXVec3TransformNormal(PG2Vec3Ref(@Self)^, PG2Vec3Ref(@Self)^, PG2MatRef(@m)^);
end;

function TG2Vec3.Len: Single;
begin
  Result := D3DXVec3Length(PG2Vec3Ref(@Self)^);
end;

function TG2Vec3.Dot(const v: TG2Vec3): Single;
begin
  Result := D3DXVec3Dot(PG2Vec3Ref(@Self)^, PG2Vec3Ref(@v)^);
end;

function TG2Vec3.Cross(const v: TG2Vec3): TG2Vec3;    
begin
  D3DXVec3Cross(PG2Vec3Ref(@Result)^, PG2Vec3Ref(@Self)^, PG2Vec3Ref(@v)^);
end;

function TG2Vec3.Angle(const v: TG2Vec3): Single;
begin
  Result := ArcCos((Self * v) / (Len * v.Len));
end;
//TG2Vec3 END

//TG2Vec4 BEGIN
class operator TG2Vec4.Negative(const v: TG2Vec4): TG2Vec4;
begin
  Result.x := -v.x;
  Result.y := -v.y;
  Result.z := -v.z;
  Result.w := -v.w;
end;

class operator TG2Vec4.Explicit(const v: TG2Vec4): TG2Vec4Ref;   
begin
  Result := PG2Vec4Ref(@v)^;
end;

class operator TG2Vec4.Explicit(const v: TG2Vec4Ref): TG2Vec4;   
begin
  Result := PG2Vec4(@v)^;
end;

class operator TG2Vec4.Implicit(const v: TG2Vec4): TG2Vec4Ref;  
begin
  Result := PG2Vec4Ref(@v)^;
end;

class operator TG2Vec4.Implicit(const v: TG2Vec4Ref): TG2Vec4;  
begin
  Result := PG2Vec4(@v)^;
end;

class operator TG2Vec4.Equal(const v1, v2: TG2Vec4): Boolean;    
begin
  Result := CompareMem(@v1, @v2, SizeOf(TG2Vec4));
end;

class operator TG2Vec4.NotEqual(const v1, v2: TG2Vec4): Boolean;    
begin
  Result := not CompareMem(@v1, @v2, SizeOf(TG2Vec4));
end;

class operator TG2Vec4.Add(const v1, v2: TG2Vec4): TG2Vec4;      
begin
  Result.x := v1.x + v2.x;
  Result.y := v1.y + v2.y;
  Result.z := v1.z + v2.z;
  Result.w := v1.w + v2.w;
end;

class operator TG2Vec4.Add(const v: TG2Vec4; const s: Single): TG2Vec4;    
begin
  Result.x := v.x + s;
  Result.y := v.y + s;
  Result.z := v.z + s;
  Result.w := v.w + s;
end;

class operator TG2Vec4.Subtract(const v1, v2: TG2Vec4): TG2Vec4;     
begin
  Result.x := v1.x - v2.x;
  Result.y := v1.y - v2.y;
  Result.z := v1.z - v2.z;
  Result.w := v1.w - v2.w;
end;

class operator TG2Vec4.Subtract(const v: TG2Vec4; const s: Single): TG2Vec4;    
begin
  Result.x := v.x - s;
  Result.y := v.y - s;
  Result.z := v.z - s;
  Result.w := v.w - s;
end;

class operator TG2Vec4.Multiply(const v1, v2: TG2Vec4): Single;      
begin
  Result := D3DXVec4Dot(PG2Vec4Ref(@v1)^, PG2Vec4Ref(@v2)^);
end;

class operator TG2Vec4.Multiply(const v: TG2Vec4; const s: Single): TG2Vec4;     
begin
  Result.x := v.x * s;
  Result.y := v.y * s;
  Result.z := v.z * s;
  Result.w := v.w * s;
end;

class operator TG2Vec4.Multiply(const v: TG2Vec4; const m: TG2Mat): TG2Vec4; 
begin
  D3DXVec4Transform(PG2Vec4Ref(@Result)^, PG2Vec4Ref(@v)^, PG2MatRef(@m)^);
end;

class operator TG2Vec4.Divide(const v: TG2Vec4; const s: Single): TG2Vec4;  
var
  RcpS: Single;
begin
  RcpS := 1 / s;
  Result.x := v.x * RcpS;
  Result.y := v.y * RcpS;
  Result.z := v.z * RcpS;
  Result.w := v.w * RcpS;
end;

procedure TG2Vec4.SetValue(const _X, _Y, _Z, _W: Single);   
begin
  x := _X; y := _Y; z := _Z; w := _W;
end;

procedure TG2Vec4.Normalize;                    
begin
  D3DXVec4Normalize(PG2Vec4Ref(@Self)^, PG2Vec4Ref(@Self)^);
end;

function TG2Vec4.Len: Single;                       
begin
  Result := D3DXVec4Length(PG2Vec4Ref(@Self)^);
end;

function TG2Vec4.Dot(const v: TG2Vec4): Single;       
begin
  Result := D3DXVec4Dot(PG2Vec4Ref(@Self)^, PG2Vec4Ref(@v)^);
end;

function TG2Vec4.Cross(const v1, v2: TG2Vec4): TG2Vec3;     
begin
  D3DXVec4Cross(PG2Vec4Ref(@Result)^, PG2Vec4Ref(@Self)^, PG2Vec4Ref(@v1)^, PG2Vec4Ref(@v2)^);
end;

function TG2Vec4.Angle(const v: TG2Vec4): Single;
begin
  Result := ArcCos((Self * v) / (Len * v.Len));
end;
//TG2Vec4 END

//TG2Quat BEGIN
class operator TG2Quat.Explicit(const q: TG2Quat): TG2QuatRef;
begin
  Result := PG2QuatRef(@q)^;
end;

class operator TG2Quat.Explicit(const q: TG2QuatRef): TG2Quat;
begin
  Result := PG2Quat(@q)^;
end;

class operator TG2Quat.Implicit(const q: TG2Quat): TG2QuatRef;     
begin
  Result := PG2QuatRef(@q)^;
end;

class operator TG2Quat.Implicit(const q: TG2QuatRef): TG2Quat;  
begin
  Result := PG2Quat(@q)^;
end;

class operator TG2Quat.Equal(const q1, q2: TG2Quat): Boolean;  
begin
  Result := CompareMem(@q1, @q2, SizeOf(TG2Quat));
end;

class operator TG2Quat.NotEqual(const q1, q2: TG2Quat): Boolean;   
begin
  Result := not CompareMem(@q1, @q2, SizeOf(TG2Quat));
end;

class operator TG2Quat.Multiply(const q1, q2: TG2Quat): TG2Quat;  
begin
  D3DXQuaternionMultiply(PG2QuatRef(@Result)^, PG2QuatRef(@q1)^, PG2QuatRef(@q2)^);
end;

procedure TG2Quat.SetValue(const _X, _Y, _Z, _W: Single);
begin
  x := _X; y := _Y; z := _Z; w := _W;
end;

procedure TG2Quat.SetIdentity;
begin
  x := 0; y := 0; z := 0; w := 1;
end;

procedure TG2Quat.SetRotation(const Axis: TG2Vec3; const Angle: Single);
begin
  D3DXQuaternionRotationAxis(PG2QuatRef(@Self)^, PG2Vec3Ref(@Axis)^, Angle);
end;

procedure TG2Quat.SetRotation(const m: TG2Mat);
begin
  D3DXQuaternionRotationMatrix(PG2QuatRef(@Self)^, PG2MatRef(@m)^);
end;

procedure TG2Quat.Rotate(const Axis: TG2Vec3; const Angle: Single);
var
  q: TG2Quat;
begin
  D3DXQuaternionRotationAxis(PG2QuatRef(@q)^, PG2Vec3Ref(@Axis)^, Angle);
  D3DXQuaternionMultiply(PG2QuatRef(@Self)^, PG2QuatRef(@Self)^, PG2QuatRef(@q)^);
end;

procedure TG2Quat.Rotate(const m: TG2Mat);
var
  q: TG2Quat;
begin
  D3DXQuaternionRotationMatrix(PG2QuatRef(@q)^, PG2MatRef(@m)^);
  D3DXQuaternionMultiply(PG2QuatRef(@Self)^, PG2QuatRef(@Self)^, PG2QuatRef(@q)^);
end;

procedure TG2Quat.Normalize;
begin
  D3DXQuaternionNormalize(PG2QuatRef(@Self)^, PG2QuatRef(@Self)^);
end;

procedure TG2Quat.GetRotation(var Axis: TG2Vec3; var Angle: Single);
begin
  D3DXQuaternionToAxisAngle(PG2QuatRef(@Self)^, PG2Vec3Ref(@Axis)^, Angle);
end;

procedure TG2Quat.GetRotation(var m: TG2Mat);
begin
  D3DXMatrixRotationQuaternion(PG2MatRef(@m)^, PG2QuatRef(@Self)^);
end;

function TG2Quat.Conjugate: TG2Quat;
begin
  D3DXQuaternionConjugate(PG2QuatRef(@Result)^, PG2QuatRef(@Self)^);
end;

function TG2Quat.Inverse: TG2Quat;
begin
  D3DXQuaternionInverse(PG2QuatRef(@Result)^, PG2QuatRef(@Self)^);
end;
//TG2Quat END

//TG2AABox BEGIN
class operator TG2AABox.Equal(const b1, b2: TG2AABox): Boolean;
begin
  Result := CompareMem(@b1, @b2, SizeOf(TG2AABox));
end;

class operator TG2AABox.NotEqual(const b1, b2: TG2AABox): Boolean;
begin
  Result := not CompareMem(@b1, @b2, SizeOf(TG2AABox));
end;

procedure TG2AABox.SetValue(const _MinV, _MaxV: TG2Vec3);
begin
  MinV := _MinV; MaxV := _MaxV;
end;
//TG2AABox END

//TG2Box BEGIN
class operator TG2Box.Equal(const b1, b2: TG2Box): Boolean;
begin
  Result := CompareMem(@b1, @b2, SizeOf(TG2Box));
end;

class operator TG2Box.NotEqual(const b1, b2: TG2Box): Boolean;  
begin
  Result := not CompareMem(@b1, @b2, SizeOf(TG2Box));
end;

class operator TG2Box.Multiply(const b: TG2Box; const m: TG2Mat): TG2Box;    
begin
  b.C.TransformCoord(m);
  b.vx.TransformNormal(m);
  b.vy.TransformNormal(m);
  b.vz.TransformNormal(m);
end;

procedure TG2Box.SetValue(const _C, _vx, _vy, _vz: TG2Vec3);
begin
  C := _C; vx := _vx; vy := _vy; vz := _vz;
end;

function TG2Box.AABox: TG2AABox;
var
  i: Int32;
  v: array[0..7] of TG2Vec3;
begin
  v[0] := C + vx + vy + vz;
  v[1] := C + vx + vy - vz;
  v[2] := C - vx + vy - vz;
  v[3] := C - vx + vy + vz;
  v[4] := C + vx - vy + vz;
  v[5] := C + vx - vy - vz;
  v[6] := C - vx - vy - vz;
  v[7] := C - vx - vy + vz;
  Result.MinV := v[0];
  Result.MaxV := v[0];
  for i := 1 to 7 do
  begin
    if v[i].x < Result.MinV.x then Result.MinV.x := v[i].x;
    if v[i].y < Result.MinV.y then Result.MinV.y := v[i].y;
    if v[i].z < Result.MinV.z then Result.MinV.z := v[i].z;
    if v[i].x > Result.MaxV.x then Result.MaxV.x := v[i].x;
    if v[i].y > Result.MaxV.y then Result.MaxV.y := v[i].y;
    if v[i].z > Result.MaxV.z then Result.MaxV.z := v[i].z;
  end;
end;
//TG2Box END

//TG2Sphere BEGIN
class operator TG2Sphere.Equal(const s1, s2: TG2Sphere): Boolean;
begin
  Result := CompareMem(@s1, @s2, SizeOf(TG2Sphere));
end;

class operator TG2Sphere.NotEqual(const s1, s2: TG2Sphere): Boolean;
begin
  Result := not CompareMem(@s1, @s2, SizeOf(TG2Sphere));
end;

procedure TG2Sphere.SetValue(const _C: TG2Vec3; const  _R: Single);
begin
  C := _C; R := _R;
end;
//TG2Sphere END

//TG2Plane BEGIN
class operator TG2Plane.Explicit(const p: TG2Plane): TG2PlaneRef;
begin
  Result := PG2PlaneRef(@p)^;
end;

class operator TG2Plane.Explicit(const p: TG2PlaneRef): TG2Plane;  
begin
  Result := PG2Plane(@p)^;
end;

class operator TG2Plane.Implicit(const p: TG2Plane): TG2PlaneRef;   
begin
  Result := PG2PlaneRef(@p)^;
end;

class operator TG2Plane.Implicit(const p: TG2PlaneRef): TG2Plane;    
begin
  Result := PG2Plane(@p)^;
end;

class operator TG2Plane.Equal(const p1, p2: TG2Plane): Boolean;
begin
  Result := CompareMem(@p1, @p2, SizeOf(TG2Plane));
end;

class operator TG2Plane.NotEqual(const p1, p2: TG2Plane): Boolean;
begin
  Result := not CompareMem(@p1, @p2, SizeOf(TG2Plane));
end;

class operator TG2Plane.Multiply(const p: TG2Plane; const m: TG2Mat): TG2Plane;  
begin
  D3DXPlaneTransform(PG2PlaneRef(@Result)^, PG2PlaneRef(@p)^, PG2MatRef(@m)^);
end;

procedure TG2Plane.SetValue(const _A, _B, _C, _D: Single);
begin
  N.SetValue(_A, _B, _C);
  D := _D;
end;

procedure TG2Plane.SetPlane(const _V, _N: TG2Vec3);
begin
  D3DXPlaneFromPointNormal(PG2PlaneRef(@Self)^, PG2Vec3Ref(@_V)^, PG2Vec3Ref(@_N)^);
end;

procedure TG2Plane.SetPlane(const _v1, _v2, _v3: TG2Vec3);
begin
  D3DXPlaneFromPoints(PG2PlaneRef(@Self)^, PG2Vec3Ref(@_v1)^, PG2Vec3Ref(@_v2)^, PG2Vec3Ref(@_v3)^);
end;

procedure TG2Plane.Normalize;
begin
  D3DXPlaneNormalize(PG2PlaneRef(@Self)^, PG2PlaneRef(@Self)^);
end;

function TG2Plane.DistanceToPoint(const v: TG2Vec3): Single;
begin
  Result := N * v - D;
end;

function TG2Plane.IntersectTri(const v1, v2, v3: TG2Vec3): Boolean;
var
  b: Boolean;
begin
  b := (N * v1 > D);
  Result := not ((b = (N * v2 > D)) and (b = (N * v3 > D)));
end;

function TG2Plane.IntersectAABox(const b: TG2AABox): Boolean;
var
  MinV, MaxV: TG2Vec3;
begin
  if N.x > 0 then
  begin
    MinV.x := b.MinV.x;
    MaxV.x := b.MaxV.x;
  end
  else
  begin
    MinV.x := b.MaxV.x;
    MaxV.x := b.MinV.x;
  end;
  if N.y > 0 then
  begin
    MinV.y := b.MinV.y;
    MaxV.y := b.MaxV.y;
  end
  else
  begin
    MinV.y := b.MaxV.y;
    MaxV.y := b.MinV.y;
  end;
  if N.z > 0 then
  begin
    MinV.z := b.MinV.z;
    MaxV.z := b.MaxV.z;
  end
  else
  begin
    MinV.z := b.MaxV.z;
    MaxV.z := b.MinV.z;
  end;
  if N * MinV + D > 0 then
  begin
    Result := False;
    Exit;
  end;
  if N * MaxV + D >= 0 then
  begin
    Result := True;
    Exit;
  end;
  Result := False;
end;

function TG2Plane.IntersectSphere(const s: TG2Sphere): Boolean;
begin
  Result := Abs(DistanceToPoint(s.C)) <= s.R;
end;
//TG2Plane END

//TG2Ray BEGIN
class operator TG2Ray.Equal(const p1, p2: TG2Plane): Boolean;
begin
  Result := CompareMem(@p1, @p2, SizeOf(TG2Ray));
end;

class operator TG2Ray.NotEqual(const p1, p2: TG2Plane): Boolean;
begin
  Result := not CompareMem(@p1, @p2, SizeOf(TG2Ray));
end;

class operator TG2Ray.Multiply(const r: TG2Ray; m: TG2Mat): TG2Ray;
begin
  r.Origin.TransformCoord(m);
  r.Dir.TransformNormal(m);
end;

procedure TG2Ray.SetValue(const _Origin, _Dir: TG2Vec3);
begin
  Origin := _Origin;
  Dir := _Dir;
end;

procedure TG2Ray.Normalize;
begin
  Dir.Normalize;
end;

procedure TG2Ray.Transform(const m: TG2Mat);
begin
  Self.Origin.TransformCoord(m);
  Self.Dir.TransformNormal(m);
end;

procedure TG2Ray.TransformInverse(const m: TG2Mat);
begin
  Transform(m.Inverse);
end;

function TG2Ray.PointAtDistance(const Distance: Single): TG2Vec3;
begin
  Result := Origin + Dir * Distance;
end;

function TG2Ray.IntersectTri(const v1, v2, v3: TG2Vec3; var U, V, Distance: Single): Boolean;
begin
  Result := D3DXIntersectTri(
    PG2Vec3Ref(@v1)^, PG2Vec3Ref(@v2)^, PG2Vec3Ref(@v3)^,
    PG2Vec3Ref(@Origin)^,
    PG2Vec3Ref(@Dir)^,
    U, V, Distance
  );
end;

function TG2Ray.IntersectSphere(const s: TG2Sphere): Boolean;
begin
  Result := D3DXSphereBoundProbe(
    PG2Vec3Ref(@s.C)^, s.R,
    PG2Vec3Ref(@Origin)^, PG2Vec3Ref(@Dir)^
  );
end;

function TG2Ray.IntersectAABox(const b: TG2AABox): Boolean;
begin
  Result := D3DXBoxBoundProbe(
    PG2Vec3Ref(@b.MinV)^, PG2Vec3Ref(@b.MaxV)^,
    PG2Vec3Ref(@Origin)^, PG2Vec3Ref(@Dir)^
  );
end;

function TG2Ray.IntersectPlane(const Plane: TG2Plane; const BackFace: Boolean; var Distance: Single): Boolean;  
var
  vd, v0: Single;
begin
  vd := Plane.N * Dir;
  if Abs(vd) < 1E-8 then
  begin
    Result := False;
    Exit;
  end;
  if not BackFace and (vd > 0) then
  begin
    Result := False;
    Exit;
  end;
  v0 := -((Plane.N * Origin) + Plane.D);
  Distance := v0 / vd;
  if Distance < 0 then
  begin
    Result := False;
    Exit;
  end;
  Result := True;
end;
//TG2Ray END

//Utility Functions BEGIN
function G2Vec2(const _X, _Y: Single): TG2Vec2;
begin
  Result.SetValue(_X, _Y);
end;

function G2Vec3(const _X, _Y, _Z: Single): TG2Vec3;
begin
  Result.SetValue(_X, _Y, _Z);
end;

function G2Vec4(const _X, _Y, _Z, _W: Single): TG2Vec4;
begin
  Result.SetValue(_X, _Y, _Z, _W);
end;

function G2Quat(const _X, _Y, _Z, _W: Single): TG2Quat;
begin
  Result.SetValue(_X, _Y, _Z, _W);
end;

function G2LerpFloat(const f1, f2, s: Single): Single;
begin
  Result := f1 + (f2 - f1) * s;
end;

function G2LerpVec2(const v1, v2: TG2Vec2; const s: Single): TG2Vec2;  
begin
  Result := v1 + (v2 - v1) * s;
end;

function G2LerpVec3(const v1, v2: TG2Vec3; const s: Single): TG2Vec3;   
begin
  Result := v1 + (v2 - v1) * s;
end;

function G2LerpVec4(const v1, v2: TG2Vec4; const s: Single): TG2Vec4;   
begin
  Result := v1 + (v2 - v1) * s;
end;

function G2SLerpQuat(const q1, q2: TG2Quat; const s: Single): TG2Quat;   
begin
  D3DXQuaternionSlerp(PG2QuatRef(@Result)^, PG2QuatRef(@q1)^, PG2QuatRef(@q2)^, s);
end;

function G2Vec2InPoly(const v: TG2Vec2; const VArr: PG2Vec2Array; const VCount: Integer): Boolean;
var
  i: Integer; 
  pi, pj: PG2Vec2; 
begin 
  Result := False; 
  if VCount < 3 then Exit;
  pj := @VArr^[VCount - 1];
  for i := 0 to VCount - 1 do
  begin 
    pi := @VArr^[i];
    if (
      ((pi^.y <= v.y) and (v.y < pj^.y))
      or
      ((pj^.y <= v.y) and (v.y < pi^.y))
    )
    and (
      v.x < (pj^.x - pi^.x) * (v.y - pi^.y) / (pj^.y - pi^.y) + pi^.x
    ) then
    Result := not Result; 
    pj := pi; 
  end; 
end;

function G2Vec2Reflect(const v, n: TG2Vec2): TG2Vec2;
var
  Dot: Single;
begin
  Dot := v * n;
  Result := G2Vec2(
    (v.x - 2 * n.x * Dot),
    (v.y - 2 * n.y * Dot)
  );
end;

procedure G2MatAddSSE(const OutM, m1, m2: PG2Mat);
asm
  movups xmm0, [edx]
  movups xmm1, [edx + 16]
  movups xmm2, [edx + 32]
  movups xmm3, [edx + 48]
  movups xmm4, [ecx]
  movups xmm5, [ecx + 16]
  movups xmm6, [ecx + 32]
  movups xmm7, [ecx + 48]
  addps xmm0, xmm4
  addps xmm1, xmm5
  addps xmm2, xmm6
  addps xmm3, xmm7
  movups [eax], xmm0
  movups [eax + 16], xmm1
  movups [eax + 32], xmm2
  movups [eax + 48], xmm3
end;

procedure G2MatAddStandard(const OutM, m1, m2: PG2Mat);
begin
  OutM^.e00 := m1.e00 + m2.e00; OutM^.e10 := m1.e10 + m2.e10; OutM^.e20 := m1.e20 + m2.e20; OutM^.e30 := m1.e30 + m2.e30;
  OutM^.e01 := m1.e01 + m2.e01; OutM^.e11 := m1.e11 + m2.e11; OutM^.e21 := m1.e21 + m2.e21; OutM^.e31 := m1.e31 + m2.e31;
  OutM^.e02 := m1.e02 + m2.e02; OutM^.e12 := m1.e12 + m2.e12; OutM^.e22 := m1.e22 + m2.e22; OutM^.e32 := m1.e32 + m2.e32;
  OutM^.e03 := m1.e03 + m2.e03; OutM^.e13 := m1.e13 + m2.e13; OutM^.e23 := m1.e23 + m2.e23; OutM^.e33 := m1.e33 + m2.e33;
end;

procedure G2MatSubSSE(const OutM, m1, m2: PG2Mat);
asm
  movups xmm0, [edx]
  movups xmm1, [edx + 16]
  movups xmm2, [edx + 32]
  movups xmm3, [edx + 48]
  movups xmm4, [ecx]
  movups xmm5, [ecx + 16]
  movups xmm6, [ecx + 32]
  movups xmm7, [ecx + 48]
  subps xmm0, xmm4
  subps xmm1, xmm5
  subps xmm2, xmm6
  subps xmm3, xmm7
  movups [eax], xmm0
  movups [eax + 16], xmm1
  movups [eax + 32], xmm2
  movups [eax + 48], xmm3
end;

procedure G2MatSubStandard(const OutM, m1, m2: PG2Mat);
begin
  OutM^.e00 := m1.e00 - m2.e00; OutM^.e10 := m1.e10 - m2.e10; OutM^.e20 := m1.e20 - m2.e20; OutM^.e30 := m1.e30 - m2.e30;
  OutM^.e01 := m1.e01 - m2.e01; OutM^.e11 := m1.e11 - m2.e11; OutM^.e21 := m1.e21 - m2.e21; OutM^.e31 := m1.e31 - m2.e31;
  OutM^.e02 := m1.e02 - m2.e02; OutM^.e12 := m1.e12 - m2.e12; OutM^.e22 := m1.e22 - m2.e22; OutM^.e32 := m1.e32 - m2.e32;
  OutM^.e03 := m1.e03 - m2.e03; OutM^.e13 := m1.e13 - m2.e13; OutM^.e23 := m1.e23 - m2.e23; OutM^.e33 := m1.e33 - m2.e33;
end;

procedure G2Mat2MulSSE(const OutM, m1, m2: PG2Mat2);
asm
  movups xmm0, [edx]
  movups xmm1, [ecx]
  shufps xmm1, xmm1, $D8 //11 01 10 00 (Transpose)
  movaps xmm2, xmm1
  shufps xmm2, xmm2, $4e //01 00 11 10 (HFlip)
  mulps xmm1, xmm0
  mulps xmm2, xmm0
  movaps xmm0, xmm1
  shufps xmm0, xmm0, $b1 //10 11 00 01 (VFlip)
  addps xmm1, xmm0
  movaps xmm0, xmm2
  shufps xmm0, xmm0, $b1 //10 11 00 01 (VFlip)
  addps xmm2, xmm0
  shufps xmm1, xmm2, $88 //10 00 10 00
  shufps xmm1, xmm1, $78 //01 11 10 00
  movups [eax], xmm1
end;

procedure G2Mat2MulStandard(const OutM, m1, m2: PG2Mat2);
var
  mr: TG2Mat2;
begin
  with mr do
  begin
    e00 := m1^.e00 * m2^.e00 + m1^.e01 * m2^.e10;
    e01 := m1^.e00 * m2^.e01 + m1^.e01 * m2^.e11;
    e10 := m1^.e10 * m2^.e00 + m1^.e11 * m2^.e10;
    e11 := m1^.e10 * m2^.e01 + m1^.e11 * m2^.e11;
  end;
  OutM^ := mr;
end;

procedure G2Vec2Mat2MulSSE(const OutV, v: PG2Vec2; const m: PG2Mat2);
asm
  movlps xmm0, [edx]
  shufps xmm0, xmm0, $44 //01 00 01 00
  movups xmm1, [ecx]
  shufps xmm1, xmm1, $D8 //11 01 10 00 (Transpose)
  mulps xmm0, xmm1
  movaps xmm1, xmm0
  shufps xmm1, xmm1, $b1 //10 11 00 01
  addps xmm0, xmm1
  shufps xmm0, xmm0, $d8 //11 01 10 00
  movlps [eax], xmm0
end;

procedure G2Vec2Mat2MulStandard(const OutV, v: PG2Vec2; const m: PG2Mat2);
var
  vr: TG2Vec2;
begin
  vr.x := v^.x * m^.e00 + v^.y * m^.e10;
  vr.y := v^.x * m^.e01 + v^.y * m^.e11;
  OutV^ := vr;
end;
//Utility Functions END

//Unit Functions BEGIN
procedure CPUExtensions;
var
  TmpEBX: UInt32;
asm
  mov [TmpEBX], EBX
  mov eax, 1
  cpuid
  test edx, 04000000h
  jz @CheckSSE
  mov [SysSSE2], 1
@CheckSSE:
  test edx, 02000000h
  jz @CheckMMX
  mov [SysSSE], 1
@CheckMMX:
  test edx, 00800000h
  jz @Done
  mov [SysMMX], 1
@Done:
  mov EBX, [TmpEBX]
end;
//Unit Functions END

initialization
  CPUExtensions;
  if SysSSE then
  begin
    G2MatAdd := G2MatAddSSE;
    G2MatSub := G2MatSubSSE;
    G2Mat2Mul := G2Mat2MulSSE;
    G2Vec2Mat2Mul := G2Vec2Mat2MulSSE;
  end
  else
  begin
    G2MatAdd := G2MatAddStandard;
    G2MatSub := G2MatSubStandard;
    G2Mat2Mul := G2Mat2MulStandard;
    G2Vec2Mat2Mul := G2Vec2Mat2MulStandard;
  end;

end.