//G2MeshLoaderG2M v1.0
//Author: Dan
unit G2MeshLoaderG2M;

{-------------------------------------------------------------------------------
G2Mesh Format:
  Header: String4 = "G2M "

  Blocks:
    Block Name: String4
    Block Size: Int4

  Block "NMAP":
    NodeCount: Int4
    NodeOffsets[NodeCount]: Int4

  Block "NDAT":
    Nodes[NodeCount]:
      NodeName: StringNT
      NodePos: Vec3f
      NodeParentID: Int4
      NodeGeomID: Int4

  Block "GMAP":
    GeomCount: Int4
    GeomOffsets[GeomCount]: Int4

  Block "GDAT":
    Geoms[GeomCount]:
      VCount: Int4
      TVCount: Int4
      FCount: Int4
      MCount: Int4;
      Vertices[VCount]: Vec3f
      TexCoords[TVCount]: Vec3f
      Materials[MCount]: Int4;
      Faces[FCount]:
        FaceVertices: Vec3i
        FaceTexCoords: Vec3i (if TVCount > 0)
        FaceGroup: Int4
        FaceMaterial: Int4

  Block "AMAP":
    AnimCount: Int4
    AnimOffsets[AnimCount]: Int4

  Block "ADAT":
    Animations[AnimCount]:
      AnimName: StringNT
      AnimStart: Int4
      AnimEnd: Int4
      AnimNodes[NodeCount]:
        NodeID: Int4
        KeyFrameCount: Int4
        KeyFrames[KeyFrameCount]:
          FrameTime: Int4
          FrameTransform: Mat4x3

  Block "SMAP":
    SkinCount: Int4
    SkinOffsets[SkinCount]: Int4

  Block "SDAT":
    Skins[SkinCount]:
      SkinGeomID: Int4
      Vertices[Geoms[SkinGeomID].VCount]:
        BoneCount: Int4
        Bones[BoneCount]:
          BoneNodeID: Int4
          BoneWeight: Float4

  Block "MMAP":
    MaterialCount: Int4
    MaterialOffsets[MaterialCount]: Int4

  Block "MDAT":
    Materials[MaterialCount]:
      MatName: StringNT;
-------------------------------------------------------------------------------}

interface

uses
  Gen2,
  G2Math,
  Windows,
  Classes,
  Controls,
  Graphics,
  PNGImage,
  SysUtils,
  SyncObjs,
  ActiveX,
  Forms,
  Math,
  Dialogs,
  DirectInput,
  DirectShow9,
  DirectMusic,
  DirectSound,
  Direct3D9,
  DXTypes,
  DXErr9,
  D3DX9;

type
  TG2MeshLoaderG2M = class (TG2MeshLoader)
  strict private
    m_InitPos: Int64;
    function ReadHeader: AnsiString; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function ReadMatrix: TG2Mat; {$IFDEF G2_USE_INLINE} inline; {$ENDIF}
    function FindBlock(const Header: AnsiString): Boolean;
  strict protected
    procedure LoadProc(const s: TStream; const MeshData: PG2MeshLoad); override;
  public
    class function GetDifinition: AnsiString; override;
    class function CanLoadStream(const s: TStream): Boolean; override;
  end;

implementation

//TG2MeshLoaderG2M BEGIN
function TG2MeshLoaderG2M.ReadHeader: AnsiString;
begin
  SetLength(Result, 4);
  m_Stream.Read(Result[1], 4);
end;

function TG2MeshLoaderG2M.ReadMatrix: TG2Mat;
var
  m4x3: array[0..3, 0..2] of Single;
  x, y: Integer;
  m: TD3DXMatrix absolute Result;
begin
  m_Stream.ReadBuffer(m4x3, SizeOf(m4x3));
  Result.SetIdentity;
  for x := 0 to 3 do
  for y := 0 to 2 do
  m.m[x, y] := m4x3[x, y];
end;

function TG2MeshLoaderG2M.FindBlock(const Header: AnsiString): Boolean;
var
  Len: Integer;
  CurHeader: AnsiString;
begin
  m_Stream.Position := m_InitPos;
  while m_Stream.Position < m_Stream.Size - 8 do
  begin
    CurHeader := ReadHeader;
    if UpperCase(CurHeader) = UpperCase(Header) then
    begin
      Result := True;
      m_Stream.Position := m_Stream.Position + 4;
      Exit;
    end
    else
    begin
      m_Stream.Read(Len, 4);
      m_Stream.Position := m_Stream.Position + Len;
    end;
  end;
  Result := False;
end;

procedure TG2MeshLoaderG2M.LoadProc(const s: TStream; const MeshData: PG2MeshLoad);
  function ReadNodes: Boolean;
  var
    NodeOffsets: array of Integer;
    i: Integer;
  begin
    Result := False;
    if FindBlock('NMAP') then
    begin
      MeshData^.NodeCount := ReadInteger;
      SetLength(MeshData^.Nodes, MeshData^.NodeCount);
      SetLength(NodeOffsets, MeshData^.NodeCount);
      m_Stream.Read(NodeOffsets[0], MeshData^.NodeCount * 4);
      if FindBlock('NDAT') then
      begin
        for i := 0 to MeshData^.NodeCount - 1 do
        begin
          m_Stream.Position := NodeOffsets[i];
          MeshData^.Nodes[i].Name := ReadString;
          MeshData^.Nodes[i].Pos := ReadVector3;
          MeshData^.Nodes[i].ParentID := ReadInteger;
          MeshData^.Nodes[i].GeomID := ReadInteger;
        end;
        Result := True;
      end
    end;
  end;
  function ReadAnims: Boolean;
  type
    TAnimNode = array of TG2Mat;
    TAnim = array of TAnimNode;
  var
    AnimOffsets: array of Integer;
    i, n, f: Integer;
    Anims: array of TAnim;
    MTranslation: TG2Vec3;
    MRotation: TG2Quat;
    MScale: TG2Vec3;
    m: TG2Mat;
    pm: PG2Mat;
  begin
    Result := False;
    if FindBlock('AMAP') then
    begin
      MeshData^.AnimCount := ReadInteger;
      SetLength(MeshData^.Anims, MeshData^.AnimCount);
      SetLength(AnimOffsets, MeshData^.AnimCount);
      SetLength(Anims, MeshData^.AnimCount);
      m_Stream.Read(AnimOffsets[0], MeshData^.AnimCount * 4);
      if FindBlock('ADAT') then
      begin
        for i := 0 to MeshData^.AnimCount - 1 do
        begin
          m_Stream.Position := AnimOffsets[i];
          MeshData^.Anims[i].Name := ReadString;
          MeshData^.Anims[i].AnimStart := ReadInteger;
          MeshData^.Anims[i].AnimEnd := ReadInteger;
          SetLength(MeshData^.Anims[i].AnimNodes, MeshData^.NodeCount);
          SetLength(Anims[i], MeshData^.NodeCount);
          for n := 0 to MeshData^.NodeCount - 1 do
          begin
            MeshData^.Anims[i].AnimNodes[n].NodeID := ReadInteger;
            MeshData^.Anims[i].AnimNodes[n].KeyFrameCount := ReadInteger;
            SetLength(MeshData^.Anims[i].AnimNodes[n].KeyFrames, MeshData^.Anims[i].AnimNodes[n].KeyFrameCount);
            SetLength(Anims[i][n], MeshData^.Anims[i].AnimNodes[n].KeyFrameCount);
            for f := 0 to MeshData^.Anims[i].AnimNodes[n].KeyFrameCount - 1 do
            begin
              MeshData^.Anims[i].AnimNodes[n].KeyFrames[f].FrameTime := ReadInteger;
              Anims[i][n][f] := ReadMatrix;
            end;
          end;
        end;
        for i := 0 to MeshData^.AnimCount - 1 do
        begin
          for n := 0 to MeshData^.NodeCount - 1 do
          begin
            for f := 0 to MeshData^.Anims[i].AnimNodes[n].KeyFrameCount - 1 do
            begin
              if MeshData^.Nodes[n].ParentID >= 0 then
              pm := @Anims[i][MeshData^.Nodes[n].ParentID][f]
              else
              pm := nil;
              m := G2MatRefine(
                MeshData^.Nodes[n].Pos,
                @Anims[i][n][f],
                pm
              );
              m.Decompose(@MScale, @MRotation, @MTranslation);
              MeshData^.Anims[i].AnimNodes[n].KeyFrames[f].FrameScaling := MScale;
              MeshData^.Anims[i].AnimNodes[n].KeyFrames[f].FrameTranslation := MTranslation;
              MeshData^.Anims[i].AnimNodes[n].KeyFrames[f].FrameRotation := MRotation;
            end;
          end;
        end;
        Result := True;
      end;
    end;
  end;
  function ReadGeoms: Boolean;
    procedure CompileGeom(const Geom: PG2MeshLoadGeom);
      procedure AddFace(const FaceID: Integer);
        procedure AddIndex(const Index: Integer);
        begin
          SetLength(Geom^.Indices, Length(Geom^.Indices) + 1);
          Geom^.Indices[High(Geom^.Indices)] := Index;
        end;
        function AddVertex(const PosID, TexID, GroupID: Integer; const Normal: TG2Vec3): DWord;
        var
          i: Integer;
          n: TG2Vec3;
          NIndArr: array of Integer;
        begin
          n := G2Vec3(0, 0, 0);
          for i := 0 to High(Geom^.Vertices) do
          if (Geom^.Vertices[i].PositionID = PosID)
          and (Geom^.Vertices[i].GroupID = GroupID) then
          begin
            if (Geom^.Vertices[i].TexCoordsID = TexID) then
            begin
              Result := i;
              Geom^.Vertices[Result].Normal := Geom^.Vertices[Result].Normal + Normal;
              Exit;
            end
            else
            begin
              n := Geom^.Vertices[i].Normal;
              SetLength(NIndArr, Length(NIndArr) + 1);
              NIndArr[High(NIndArr)] := i;
            end;
          end;
          n := n + Normal;
          for i := 0 to High(NIndArr) do
          Geom^.Vertices[NIndArr[i]].Normal := n;
          Result := Length(Geom^.Vertices);
          SetLength(Geom^.Vertices, Length(Geom^.Vertices) + 1);
          Geom^.Vertices[Result].PositionID := PosID;
          Geom^.Vertices[Result].TexCoordsID := TexID;
          Geom^.Vertices[Result].GroupID := GroupID;
          Geom^.Vertices[Result].Normal := n;
        end;
      var
        i: Integer;
        Normal: TG2Vec3;
      begin
        Normal := G2TriangleNormal(
          Geom^.Positions[Geom^.Faces[FaceID].FacePositions[0]],
          Geom^.Positions[Geom^.Faces[FaceID].FacePositions[1]],
          Geom^.Positions[Geom^.Faces[FaceID].FacePositions[2]]
        );
        for i := 0 to 2 do
        AddIndex(
          AddVertex(
            Geom^.Faces[FaceID].FacePositions[i],
            Geom^.Faces[FaceID].FaceTexCoords[i],
            Geom^.Faces[FaceID].FaceGroup,
            Normal
          )
        );
      end;
    var
      i: Integer;
    begin
      for i := 0 to Geom^.FCount - 1 do
      AddFace(i);
      for i := 0 to High(Geom^.Vertices) do
      Geom^.Vertices[i].Normal.Normalize;
    end;
  var
    GeomOffsets: array of Integer;
    i: Integer;
  begin
    Result := False;
    if FindBlock('GMAP') then
    begin
      MeshData^.GeomCount := ReadInteger;
      SetLength(MeshData^.Geoms, MeshData^.GeomCount);
      SetLength(GeomOffsets, MeshData^.GeomCount);
      m_Stream.Read(GeomOffsets[0], MeshData^.GeomCount * 4);
      if FindBlock('GDAT') then
      begin
        for i := 0 to MeshData^.GeomCount - 1 do
        begin
          m_Stream.Position := GeomOffsets[i];
          MeshData^.Geoms[i].VCount := ReadInteger;
          MeshData^.Geoms[i].TVCount := ReadInteger;
          MeshData^.Geoms[i].FCount := ReadInteger;
          MeshData^.Geoms[i].MCount := ReadInteger;
          SetLength(MeshData^.Geoms[i].Positions, MeshData^.Geoms[i].VCount);
          SetLength(MeshData^.Geoms[i].TexCoords, MeshData^.Geoms[i].TVCount);
          SetLength(MeshData^.Geoms[i].Faces, MeshData^.Geoms[i].FCount);
          SetLength(MeshData^.Geoms[i].Materials, MeshData^.Geoms[i].MCount);
          m_Stream.Read(MeshData^.Geoms[i].Positions[0], MeshData^.Geoms[i].VCount * 12);
          m_Stream.Read(MeshData^.Geoms[i].TexCoords[0], MeshData^.Geoms[i].TVCount * 12);
          m_Stream.Read(MeshData^.Geoms[i].Materials[0], MeshData^.Geoms[i].MCount * 4);
          if MeshData^.Geoms[i].TVCount > 0 then
          m_Stream.Read(MeshData^.Geoms[i].Faces[0], SizeOf(TG2MeshLoadGeomFace) * MeshData^.Geoms[i].FCount)
          else
          m_Stream.Read(MeshData^.Geoms[i].Faces[0], (SizeOf(TG2MeshLoadGeomFace) - 12) * MeshData^.Geoms[i].FCount);
          CompileGeom(@MeshData^.Geoms[i]);
          MeshData^.Geoms[i].SkinID := -1;
          MeshData^.Geoms[i].OptPreNormal := True;
          MeshData^.Geoms[i].OptPreTangent := False;
        end;
        Result := True;
      end;
    end;
  end;
  function ReadSkins: boolean;
    function AddBone(const SkinID: Integer; const BoneID: Integer): Integer;
    var
      b: Integer;
    begin
      for b := 0 to High(MeshData^.Skins[SkinID].Bones) do
      if BoneID = MeshData^.Skins[SkinID].Bones[b] then
      begin
        Result := b;
        Exit;
      end;
      Result := Length(MeshData^.Skins[SkinID].Bones);
      SetLength(MeshData^.Skins[SkinID].Bones, Length(MeshData^.Skins[SkinID].Bones) + 1);
      MeshData^.Skins[SkinID].Bones[High(MeshData^.Skins[SkinID].Bones)] := BoneID;
    end;
  var
    SkinOffsets: array of Integer;
    i, v, b: Integer;
  begin
    Result := False;
    if FindBlock('SMAP') then
    begin
      MeshData^.SkinCount := ReadInteger;
      SetLength(MeshData^.Skins, MeshData^.SkinCount);
      SetLength(SkinOffsets, MeshData^.SkinCount);
      m_Stream.Read(SkinOffsets[0], MeshData^.SkinCount * 4);
      if FindBlock('SDAT') then
      begin
        for i := 0 to MeshData^.SkinCount - 1 do
        begin
          m_Stream.Position := SkinOffsets[i];
          MeshData^.Skins[i].GeomID := ReadInteger;
          MeshData^.Geoms[MeshData^.Skins[i].GeomID].SkinID := i;
          SetLength(MeshData^.Skins[i].Vertices, MeshData^.Geoms[MeshData^.Skins[i].GeomID].VCount);
          for v := 0 to MeshData^.Geoms[MeshData^.Skins[i].GeomID].VCount - 1 do
          begin
            MeshData^.Skins[i].Vertices[v].BoneCount := ReadInteger;
            SetLength(MeshData^.Skins[i].Vertices[v].BoneIndices, MeshData^.Skins[i].Vertices[v].BoneCount);
            SetLength(MeshData^.Skins[i].Vertices[v].BoneWeights, MeshData^.Skins[i].Vertices[v].BoneCount);
            for b := 0 to MeshData^.Skins[i].Vertices[v].BoneCount - 1 do
            begin
              MeshData^.Skins[i].Vertices[v].BoneIndices[b] := AddBone(i, ReadInteger);
              MeshData^.Skins[i].Vertices[v].BoneWeights[b] := ReadFloat;
            end;
          end;
          MeshData^.Skins[i].BoneCount := Length(MeshData^.Skins[i].Bones);
        end;
        Result := True;
      end;
    end;
  end;
  function ReadMaterials: Boolean;
  var
    i: Integer;
    MaterialOffsets: array of Integer;
  begin
    Result := False;
    if FindBlock('MMAP') then
    begin
      MeshData^.MaterialCount := ReadInteger;
      SetLength(MeshData^.Materials, MeshData^.MaterialCount);
      SetLength(MaterialOffsets, MeshData^.MaterialCount);
      m_Stream.Read(MaterialOffsets[0], MeshData^.MaterialCount * 4);
      if FindBlock('MDAT') then
      begin
        for i := 0 to MeshData^.MaterialCount - 1 do
        begin
          m_Stream.Position := MaterialOffsets[i];
          MeshData^.Materials[i].Name := ReadString;
        end;
      end;
    end;
  end;
begin
  if ReadHeader <> 'G2M ' then Exit;
  m_InitPos := m_Stream.Position;
  ReadNodes;
  ReadAnims;
  ReadGeoms;
  ReadSkins;
  ReadMaterials;
end;

class function TG2MeshLoaderG2M.GetDifinition: AnsiString;
begin
  Result := 'Gen2 Mesh Loader';
end;

class function TG2MeshLoaderG2M.CanLoadStream(const s: TStream): Boolean;
var
  Def: AnsiString;
  Pos: Int64;
begin
  Pos := s.Position;
  SetLength(Def, 4);
  s.Read(Def[1], 4);
  s.Position := Pos;
  Result := Def = 'G2M ';
end;
//TG2MeshLoaderG2M END

initialization
  G2RegMeshLoader(TG2MeshLoaderG2M);

end.