program Arcanoid;

uses
  sNail,
  Windows,
  smXmodel,
  OpenGL,
  uMath,
  uGL,
  uGameEngine,
  uParticles,
  uGameObj,
  uMenu;

//{$R *.res}
var
  factor:single;
  fullscreen,vsync:boolean;
  
procedure Preload;
const
  Diffuse:array[0..3] of single = (1.0,1.0,1.0,1.0);
  Specular:array[0..3] of single = (1.0,1.0,1.0,1.0);
begin
  pack_SetPassword('igdc');
  tex_Load(PACK_TEX,'techfont.bmp',tex_TechFnt,false);

  glEnable(GL_COLOR_MATERIAL);
  glEnable(GL_NORMALIZE);
  glEnable(GL_LIGHT0);
  gl_LightDefault(GL_LIGHT0);

  glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL_EXT, GL_SEPARATE_SPECULAR_COLOR_EXT);

  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, @Diffuse[0]);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, @Specular[0]);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 32);

  glCullFace(GL_BACK);
  gl_SetOrtho;
  RenderText24(10,10,'Loading...');
end;

procedure EngineInit;
begin
  snd_Init;
  LoadGameResurces;
  GameEng:=TGameEngine.Create;
  CreateMenu;
  pack_Reset;
end;

procedure EngineUpdate;
begin
  factor:=snail_GetCPUTick/1600;

  if inMenu
  then
    UpdateMenu
  else
    GameEng.Update;
end;

procedure RenderBG;
var
  i:byte;
begin
  glColor4f(sin(factor),1,cos(factor),0.5+sin(factor)/4);

  glPushMatrix;
  gl_TexEnable(tex_bground);
     gl_RenderTex(0,0,gl_GetResY,gl_GetResY,0);
  gl_TexDisable;
  glPopMatrix;

  glColor3f(1,1,1);
  gl_TexEnable(tex_border);
  
  for i:=0 to (gl_GetResY shr 4) do
  begin
    glPushMatrix;
    gl_RenderTex(gl_GetResY,i*16,8,16,0);
    glPopMatrix;
  end;
  gl_TexDisable;
end;

procedure EngineRender;
begin
  gl_ClearScreen(true,true,false);
  gl_SetOrtho;
  gl_SetBlend(2);
  RenderBG;

  gl_LightPos(GL_LIGHT0,cos(factor),sin(factor),1.5);
  if inMenu then
    RenderMenu
  else
    GameEng.Render;
end;

procedure EngineFree;
begin
  FreeGameResurces;
  GameEng.Free;
end;

begin
  Randomize;
  ShowCursor(false);
  tex_Params(FILTER_BILINEAR,false);

  snail_RegProc(PROC_PRELOAD,@Preload);
  snail_RegProc(PROC_INIT,@EngineInit);
  snail_RegProc(PROC_UPDATE,@EngineUpdate);
  snail_RegProc(PROC_RENDER,@EngineRender);
  snail_RegProc(PROC_FREE,@EngineFree);

  snail_SetTimer(18);
  snail_SetName('Arcanoid-2');

  fullscreen:=ini_ReadBool(CONFIG_INI,'options','fullscreen');
  vsync:=ini_ReadBool(CONFIG_INI,'options','vsync');

  snail_Run(800,600,32,75,0,fullscreen,vsync,true);
  ShowCursor(true);
end.
