unit smXmodel;

interface
uses Windows,sNail,uMath,uGL,OpenGL;

type
  TTexInfo= record
    ID:DWORD;
    TexName:string;
  end;

  TMesh = record
    TexID:integer;
    VCount,FCount:word;
    Vertex:array of TVector;
    Face:array of TFace;
    TexVertex:array of TVector2D;
    Normal:array of TVector;
  end;

  TStaticModel = class
    Loaded:boolean;
    MeshCount,TexCount:word;
    PolyCount:cardinal; 
    Mesh:array of TMesh;
    Textures:array of TTexInfo;
  public
    ScaleValue:single;
    procedure Clear;
    procedure LoadFromFile(const FileName:string;Scale:boolean);
    procedure LoadTextures;
    procedure Render;
    procedure CalcNormal;
    procedure ToScale;
  end;

implementation
uses uGameEngine;
procedure TStaticModel.ToScale;
var
  i,j:word;
  MaxV:single;
begin
  MaxV:=0;
  for i:=0 to MeshCount - 1 do
    for j:=0 to Mesh[i].VCount - 1 do
    with Mesh[i] do
    begin
      MaxV:=Max(MaxV,Vertex[j].X);
      MaxV:=Max(MaxV,Vertex[j].Y);
      MaxV:=Max(MaxV,Vertex[j].Z);
    end;
end;

procedure TStaticModel.Clear;
var
  i:byte;
begin
  if TexCount > 0 then
    for i:=0 to TexCount-1 do
      with Textures[i] do
        tex_Free(ID);

  TexCount:=0;
  Loaded:=false;
  MeshCount:=0;
  PolyCount:=0;
  SetLength(Mesh,0);
  SetLength(Textures,0);
end;

procedure TStaticModel.LoadTextures;
var
  i:word;
begin
  if TexCount > 0 then
    for i:=0 to TexCount-1 do
      with Textures[i] do
        tex_Load(PACK_TEX,PChar(TexName),ID,true);
end;

procedure TStaticModel.LoadFromFile(const FileName:string;Scale:boolean);
var
  e:array[0..2] of char;
  F:file of byte;
  i:integer;
  name_len:byte;
const
  ext = 'SMK';
begin
  Clear;

  AssignFile(F,FileName);
  Reset(F);
  BlockRead(F,e,3);
  if e <> ext then
  begin
    CloseFile(F);
    exit;
  end;

  BlockRead(F,MeshCount,sizeof(MeshCount));
  SetLength(Mesh,MeshCount);
  BlockRead(F,TexCount,sizeof(TexCount));
  SetLength(Textures,TexCount);

  if TexCount > 0 then
    for i:=0 to TexCount - 1 do
    begin
      BlockRead(F,name_len,sizeof(name_len));
      SetLength(Textures[i].TexName,name_len);
      BlockRead(F,Textures[i].TexName[1],name_len);
    end;

  for i:=0 to MeshCount-1 do
  with Mesh[i] do
  begin
    BlockRead(F,VCount,sizeof(VCount));
    BlockRead(F,FCount,sizeof(FCount));
    BlockRead(F,TexID,sizeof(TexID));

    SetLength(Vertex,VCount);
    SetLength(Face,FCount);
    SetLength(TexVertex,VCount);

    BlockRead(F,Vertex[0],VCount*sizeof(TVector));
    BlockRead(F,Face[0],FCount*sizeof(TFace));
    BlockRead(F,TexVertex[0],VCount*sizeof(TVector2D));
    inc(PolyCount,FCount);
  end;

  CloseFile(F);
  CalcNormal;
  if Scale then ToScale;
  LoadTextures;
  Loaded:=true;
  log_AddStr(nil,PChar('- Model > Load Successful: "'+FileName+'"'));
end;

procedure TStaticModel.Render;
var
  i:word;
begin
  if not Loaded then Exit;

  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);

  glScalef(ScaleValue,ScaleValue,ScaleValue);

  for i:=0 to MeshCount-1 do
  with Mesh[i] do
  begin
    gl_TexEnable(Self.Textures[TexID].ID);
    
    glPushMatrix;
      glNormalPointer(GL_FLOAT,0,Normal);
      glVertexPointer(3,GL_FLOAT,0,Vertex);
      glTexCoordPointer(2,GL_FLOAT,0,TexVertex);

      glDrawElements(GL_TRIANGLES,FCount*3,GL_UNSIGNED_SHORT,Face);
    glPopMatrix;
  end;
  gl_TexDisable;

  glDisableClientState(GL_VERTEX_ARRAY);
  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
end;

procedure TStaticModel.CalcNormal;
var
  i,j,f:integer;
  n:TVector;
begin
  for i:=0 to MeshCount - 1 do
    with Mesh[i] do
    begin
      SetLength(Normal,VCount);
      FillChar(Normal[0],VCount*SizeOf(TVector), 0);

      for j:=0 to FCount-1 do
      begin
        n:=Plane_Normal(Vertex[Face[j][2]],
                        Vertex[Face[j][1]],
                        Vertex[Face[j][0]]);
        for f:=0 to 2 do
          Normal[Face[j][f]]:=Vec_Sub(Normal[Face[j][f]],n);
      end;

      for j:=0 to VCount - 1 do
        Normal[j]:=Vec_Norm(Normal[j]);
  end;
end;

end.
