unit uGameEngine;

interface
uses sNail,Windows,OpenGL,uGameObj,uParticles,uMenu,uGL;
const
  PACK_TEX = 'data\texs.mpx';
  PACK_SND = 'data\snds.mpx';
  
type
  TGameEngine = class
  private
    Score:longint;
    procedure LoadLevel(FileName:string);
    procedure RenderHud;
    
  public
    Obj3DCount:word;
    Objects3D:array of TGameObj3D;
    Particles:TPartEngine;
    Hint:TSplashHint;
    
    Level:word;
    Lives:byte;
    GameOver:boolean;

    constructor Create;
    procedure ResetGame;
    procedure ResetObjs;
    procedure NextLevel;
    procedure SetScore(Value:byte);
    procedure AddObj3D(ObjType:byte;X,Y:single);
    procedure DelObj3D(i:word);

    procedure Update;
    procedure Render;
  end;

var
  GameEng:TGameEngine;
  MaxLevels:word;

implementation
const
  PATH_LEVELS = 'data\levels\';
{ TGameEngine }

constructor TGameEngine.Create;
begin
  GameOver:=true;
  AddObj3D(OBJ_3D_BAT,0,gl_GetResY-30);
  AddObj3D(OBJ_3D_BALL,100,gl_GetResY-40);
  Particles:=TPartEngine.Create;
  Particles.Add(PART_FIRE,20,100,$ffff00);
  MaxLevels:=ini_ReadInt(CONFIG_INI,'levels','count');
end;

procedure TGameEngine.ResetGame;
begin
  GameOver:=false;
  Score:=0;
  Level:=0;
  Lives:=3;
  MaxBricks:=0;
  NextLevel;
  inp_SetMousePos(gl_GetResY shr 1-10,gl_GetResY shr 1);
end;

procedure TGameEngine.AddObj3D(ObjType: byte; X, Y: single);
begin
  SetLength(Objects3D,Length(Objects3D)+1);
  case ObjType of
    OBJ_3D_BAT:Objects3D[Obj3DCount]:=TBat.Create(X,Y);
    OBJ_3D_BALL:Objects3D[Obj3DCount]:=TBall.Create(X,Y);
    OBJ_3D_BRICK:Objects3D[Obj3DCount]:=TBrick.Create(X,Y);
  end;
  inc(Obj3DCount);
end;

procedure TGameEngine.DelObj3D(i:word);
begin
  Objects3D[i].Free;
  Objects3D[i]:=nil;
end;

procedure TGameEngine.LoadLevel(FileName: string);
const
  MAX_CELLS = 15;
var
  F:File of byte;
  CellX,CellY:byte;
  Bricks:array[0..MAX_CELLS,0..MAX_CELLS] of byte;
  i,j:byte;
  StepX,StepY:single;
begin
  AssignFile(F,PATH_LEVELS+FileName);
  Reset(F);

  BlockRead(F,CellX,sizeof(CellX)); // - ,
  BlockRead(F,CellY,sizeof(CellY)); //  ...
  //  ...
  BlockRead(F,Bricks[0,0],sqr(MAX_CELLS)*sizeof(Bricks[0,0]));

  StepX:=(gl_GetResX / CellX)*1.2;
  StepY:=(gl_GetResY / CellY)*1.2;

  Obj3DCount:=2;
  SetLength(Objects3D,2);
  
  for i:=0 to CellX - 1 do
    for j:=0 to CellY - 1 do
    begin
      if Bricks[i,j] in [1..6] then
      begin
        AddObj3D(OBJ_3D_BRICK,((StepX/2)*i)+StepX,((StepY/2)*j)+StepY);
        with (Objects3D[High(Objects3D)] as TBrick) do
          SetType(Bricks[i,j]);
      end;
    end;

  CloseFile(F);
  log_AddStr(nil,PChar('- Level > Loaded : "'+FileName+'"'));
  ResetObjs;
  Hint.Go('Level '+Int2Str(Level));
end;

procedure TGameEngine.Render;
var
  i:word;
begin
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);
  glEnable(GL_CULL_FACE);

  gl_SetPerspective(45,0.1,100);
  gl_SetOrtho3D;

  for i:=0 to Obj3DCount-1 do
    if Objects3D[i].Active then Objects3D[i].Render;

  glPopMatrix;

  glDisable(GL_CULL_FACE);
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);

  gl_SetOrtho;
  RenderHud;
  Particles.Render;
  Hint.Render;

  if GameOver then
    RenderText24(25,450,'Press ESC for return in main menu.');
end;

procedure TGameEngine.Update;
var
  i:word;
begin
  if inp_KeyPress(VK_ESCAPE) then
  begin
    InMenu:=true;
    CurBtn:=-1;
  end;

  if inp_GetMousePos.X < 5 then inp_SetMousePos(5,inp_getMousePos.Y);
  if inp_GetMousePos.Y < (gl_GetResY shr 1) then inp_SetMousePos(inp_GetMousePos.X,(gl_GetResY shr 1));

  if inp_GetMousePos.X > gl_GetResY then inp_SetMousePos(gl_GetResY,inp_getMousePos.Y);
  if inp_GetMousePos.Y > gl_GetResY-5 then inp_SetMousePos(inp_GetMousePos.X,gl_GetResY-5);

  for i:=0 to Obj3DCount-1 do
  begin
    if Objects3D[i].Active then
      Objects3D[i].Update;
  end;

  Particles.Update;
  Hint.Update;
end;

procedure TGameEngine.RenderHud;
var
  i:byte;
begin
  gl_SetColor($ffffff);
  gl_TexEnable(tex_score);
  for i:=0 to 2 do
  begin
    glPushMatrix;
      gl_RenderTex(RightX,50+(i*100),128,64,0);
    glPopMatrix;
  end;
  gl_TexDisable;

  RenderText32(RightX,20,'Score');
  RenderText32(RightX,120,'Lives ');
  RenderText32(RightX,220,'Level ');
  RenderText32(RightX+GetScoreCenter(Score),65,Int2Str(Score),$C00000);
  RenderText32(RightX+GetScoreCenter(Lives),165,Int2Str(Lives),$00C000);
  RenderText32(RightX+GetScoreCenter(Level),265,Int2Str(Level),$0000C0);

  gl_TexDisable;
end;

procedure TGameEngine.SetScore(Value: byte);
begin
  Inc(Score,Value);
end;

procedure TGameEngine.NextLevel;
begin
  Inc(Level);
  if Level > MaxLevels then
  begin
    GameOver:=true;
    exit;
  end;
  LoadLevel('level'+Int2Str(Level)+'.arc');
end;

procedure TGameEngine.ResetObjs;
begin
  (Objects3D[OBJ_3D_BAT] as TBat).Angle:=0;
  (Objects3D[OBJ_3D_BALL] as TBall).Reset;
end;

end.
