unit uGameObj;

interface
uses uMath,smXmodel,OpenGL,sNail,uGL;
const
  OBJ_3D_BAT = 0;
  OBJ_3D_BALL = 1;
  OBJ_3D_BRICK = 2;
type
  TGameObj3D = class
  public
    ObjID:byte;
    Pos:TVector2D;
    Active:boolean;
    constructor Create(X,Y:single);virtual;
    procedure Update;virtual;
    procedure Render;virtual;
    procedure Free;virtual;
  end;

  TBrick = class(TGameObj3D)
  private
    MeshId,BType:byte;
    Color:integer;
    Health,Points:byte;
  public
    procedure SetType(bType:byte);
    procedure SetParams(MeshId:byte;Color:integer;Health:byte);
    constructor Create(X,Y:single);override;
    procedure Render;override;
    procedure CollideAction;
  end;

  TBat = class(TGameObj3D)
  private
    NormY:single;
  public
    Angle:single;
    constructor Create(X,Y:single);override;
    procedure Update;override;
    procedure Render;override;
  end;

  TBall = class(TGameObj3D)
  private
    SpeedX,SpeedY,StartY:single;
    Fly:boolean;
  public
    constructor Create(X,Y:single);override;
    procedure Update;override;
    procedure Render;override;
    procedure Reset;
  end;
  
  TSplashHint = object
    X,Y:single;
    Text:string;
    Alpha:byte;
    Time:single;
    Active,Back:boolean;
    procedure Go(S:string);
    procedure Update;
    procedure Render;
  end;

  procedure LoadGameResurces;
  procedure FreeGameResurces;

var
  tex_bground,tex_score,tex_border:cardinal;
  snd_Hit,snd_BatKick,snd_BallFire:integer;
  mBricks:array[0..2] of TStaticModel;
  mBat,mBall:TStaticModel;
  BallPos:^TVector2D;
  MaxBricks:smallint = 0;

implementation
uses uGameEngine,uParticles,uMenu;
const
  PATH_MODELS = 'data\models\';
  BSCALE:array[0..2] of single = (0.185,0.025,0.185);

procedure LoadGameResurces;
var
  i:byte;
begin
  tex_Load(PACK_TEX,'bground.jpg',tex_bground,false);
  tex_Load(PACK_TEX,'border.gif',tex_border,false);
  tex_Load(PACK_TEX,'logo.png',tex_logo,false);
  tex_Load(PACK_TEX,'button.png',tex_button,false);
  tex_Load(PACK_TEX,'score.png',tex_score,false);
  tex_Load(PACK_TEX,'controls.png',tex_controls,false);
  tex_Load(PACK_TEX,'cursor.png',tex_cursor,false);
  tex_Load(PACK_TEX,'part.bmp',tex_part,false);
  // Load Bricks Models ...
  for i:=0 to High(mBricks) do
  begin
    mBricks[i]:=TStaticModel.Create;
    mBricks[i].LoadFromFile(PATH_MODELS+'brick'+Int2Str(i+1)+'.smk',true);
    mBricks[i].ScaleValue:=BSCALE[i];
  end;
  // load player mesh...
  mBat:=TStaticModel.Create;
  mBat.LoadFromFile(PATH_MODELS+'krogat.smk',true);
  mBat.ScaleValue:=0.0091;

  mBall:=TStaticModel.Create;
  mBall.LoadFromFile(PATH_MODELS+'ball.smk',true);
  mBall.ScaleValue:=0.25;

  snd_Hit:=snd_Load(PACK_SND,'hit.wav');
  snd_BatKick:=snd_Load(PACK_SND,'bat_kick.wav');
  snd_BallFire:=snd_Load(PACK_SND,'ball_fire.wav');
end;

procedure FreeGameResurces;
var
  i:byte;
begin
  // Bricks Free ...
  for i:=0 to High(mBricks) do
    mBricks[i].Clear;

  mBat.Clear;
  mBall.Clear;
end;

{ TGameObj }
constructor TGameObj3D.Create(X, Y: single);
begin
  Pos.X:=X;
  Pos.Y:=Y;
  Active:=true;
end;

procedure TGameObj3D.Free;
begin
end;
procedure TGameObj3D.Render;
begin

end;
procedure TGameObj3D.Update;
begin
end;

{ TBrick }

constructor TBrick.Create(X, Y: single);
begin
  inherited Create(X,Y);
  ObjID:=OBJ_3D_BRICK;
  SetParams(0,$ff0000,1);
end;

procedure TBrick.CollideAction;
begin
  Dec(Health);

  with GameEng do
  begin
    Particles.Add(PART_BOOM,Pos.X,Pos.Y,Color);
    snd_Play(snd_Hit);
    if Health = 0 then
    begin
      Dec(MaxBricks);
      Active:=false;
      GameEng.SetScore((10*Points) div 3);
    end;

    if MaxBricks <= 0 then GameEng.NextLevel;
  end;  
end;

procedure TBrick.SetParams(MeshId:byte; Color:integer; Health:byte);
begin
  Self.MeshId:=MeshId;
  Self.Color:=Color;
  Self.Health:=Health;
  Points:=Health;
end;

procedure TBrick.SetType(bType: byte);
begin
  if bType in [1..4,6] then inc(MaxBricks);
  Self.BType:=bType;
  case bType of
    1:SetParams(0,$ff0000,1);
    2:SetParams(0,$00ff00,1);
    3:SetParams(0,$0000ff,1);
    4:SetParams(0,$ff00ff,1);
    5:SetParams(1,$808080,32);
    6:SetParams(2,$ffff00,3);
  end;
end;

procedure TBrick.Render;
begin
  inherited Render;

  gl_SetColor(Color);
  glPushMatrix;
    gl_Position2D(Pos.X,Pos.Y,0);
    mBricks[MeshId].Render;
  glPopMatrix;
end;


{ TBat }

constructor TBat.Create(X, Y:single);
begin
  inherited Create(X,Y);
  ObjID:=OBJ_3D_BAT;
  Angle:=0.0;
  NormY:=Pos.Y;
end;

procedure TBat.Render;
begin
  inherited Render;
  gl_SetColor($ffffff);
  glPushMatrix;
    gl_Position2D(Pos.X,Pos.Y,0);
    glRotatef(Angle,0,0,1);
    mBat.Render;
  glPopMatrix;
end;

procedure TBat.Update;
begin
  inherited Update;

  if inp_MousePress(MOUSE_BTN_LEFT) then
    if Angle < 12.0 then
      Angle:=Angle+0.8;

  if inp_MousePress(MOUSE_BTN_RIGHT) then
    if Angle > -12.0 then
      Angle:=Angle-0.8;
      
  if inp_MousePress(MOUSE_BTN_LEFT) and inp_MousePress(MOUSE_BTN_RIGHT) then
  if Angle <> 0 then
  begin
    if Angle > 0 then Angle:=Angle-0.8;
    if Angle < 0 then Angle:=Angle+0.8;
  end;

  Pos.X:=inp_GetMousePos.X;
  if Pos.X < 36 then Pos.X:=38;
  if Pos.X > gl_GetResY - 38 then Pos.X:=gl_GetResY - 38;
  if Pos.Y > NormY then Pos.Y:=Pos.Y-1.0;
end;

{ TBall }

constructor TBall.Create(X, Y: single);
begin
  inherited Create(X,Y);
  StartY:=Y;
  ObjID:=OBJ_3D_BALL;
  BallPos:=@Pos;
end;

procedure TBall.Render;
begin
  inherited;
  gl_SetColor($ffffff);
  glPushMatrix;
    gl_Position2D(Pos.X,Pos.Y,0);
    mBall.Render;
  glPopMatrix;
end;

procedure TBall.Reset;
begin
  Pos.Y:=StartY;
  Pos.X:=-10;
  SpeedY:=4.8;
  Fly:=false;
end;

procedure TBall.Update;
const
  RADIUS = 6;
  BRICK_WIDTH = 18;
  BRICK_HEIGHT = 12;
var
  i:word;
  ColCount:byte;
  DistX,DistY:single;
  BrickPos:TVector2D;
begin
  inherited;

  with GameEng do
  begin
    if GameOver then exit;

    if not Fly then
    begin
      Pos.X:=Objects3D[OBJ_3D_BAT].Pos.X;
      if inp_MouseClick(MOUSE_BTN_MIDDLE) then
      begin
        Fly:=true;
        SpeedX:=(Objects3D[OBJ_3D_BAT] as TBat).Angle/pi;
        Particles.Add(PART_BOOM,Pos.X,Pos.Y,$ffff00);
        snd_Play(snd_BallFire);
      end;
      exit;
    end;

    Pos.X:=Pos.X-SpeedX;
    Pos.Y:=Pos.Y-SpeedY;

    if (Pos.X <= 0) or (Pos.X >= gl_GetResY) then
      SpeedX:=-SpeedX;

    if (Pos.Y <= 0) then
      SpeedY:=-SpeedY;

    if (Pos.Y > gl_GetResY) then
    begin
      Dec(Lives);
      if Lives <> 0 then
        Hint.Go('Oops!')
      else
      begin
        Hint.Go('Game Over!');
        GameOver:=true;
      end;
      
      Reset;
    end;

    if (Vec2D_Dist(Objects3D[0].Pos,Vec2D(Pos.X,Pos.Y-25)) < 40) and (Pos.Y < Objects3D[0].Pos.Y+15) then
    begin
      Objects3D[OBJ_3D_BAT].Pos.Y:=Objects3D[OBJ_3D_BAT].Pos.Y+6.0;
      SpeedX:=(Objects3D[0] as TBat).Angle/pi;
      SpeedY:=-SpeedY;
      GameEng.Particles.Add(1,Pos.X,Pos.Y,$ffff00);
      snd_Play(snd_BatKick);
    end;

    for i:=2 to GameEng.Obj3DCount-1 do
      if Objects3D[i].Active then
       begin
         ColCount:=0;
         BrickPos:=Objects3D[i].Pos;

         // top ..
         if (Pos.X >= (BrickPos.X-BRICK_WIDTH)) and (Pos.X <= (BrickPos.X+BRICK_WIDTH)) then
         begin
           DistY:=Pos.Y - (BrickPos.Y-BRICK_HEIGHT);
           if abs(DistY) < RADIUS then
           begin
             SpeedY:=-SpeedY;
             Pos.Y:=BrickPos.Y-BRICK_HEIGHT-RADIUS;
             Inc(ColCount);
           end;
           // bottom ...
           DistY:=Pos.Y - (BrickPos.Y+BRICK_HEIGHT);
           if abs(DistY) < RADIUS then
           begin
             SpeedY:=-SpeedY;
             Pos.Y:=BrickPos.Y+BRICK_HEIGHT+RADIUS;
             Inc(ColCount);
           end;
         end;

         if (Pos.Y >= (BrickPos.Y-BRICK_HEIGHT)) and (Pos.Y <= (BrickPos.Y+BRICK_HEIGHT)) then
         begin
           // left...
           DistX:=Pos.X - (BrickPos.X-BRICK_WIDTH);
           if abs(DistX) < RADIUS then
           begin
             SpeedX:=-SpeedX;
             Pos.X:=BrickPos.X-BRICK_WIDTH-RADIUS;
             Inc(ColCount);
           end;
           // right ...
           DistX:=Pos.X - (BrickPos.X+BRICK_WIDTH);
           if abs(DistX) < RADIUS then
           begin
             SpeedX:=-SpeedX;
             Pos.X:=BrickPos.X+BRICK_WIDTH+RADIUS;
             Inc(ColCount);
           end;
         end;
         if ColCount > 0 then
         begin
           (Objects3D[i] as TBrick).CollideAction;
           break;
         end;
       end;
    end;
end;

{ TSplashHint }

procedure TSplashHint.Go(S: string);
begin
  Text:=S;
  X:=0;
  Y:=10;
  Alpha:=0;
  Time:=0;
  Back:=false;
  Active:=true;
end;

procedure TSplashHint.Render;
begin
  if not Active then exit;
  gl_TexEnable(tex_TechFnt);
    glPushMatrix;
      RenderText32(x,y,Text,$ffffff,Alpha);
    glPopMatrix;
  gl_TexDisable;
end;

procedure TSplashHint.Update;
begin
  if not Active then exit;
  if not Back then
  begin
    Time:=Time+0.1;
    if Alpha < 210 then
    begin
      X:=X+1;
      Alpha:=Alpha+5;
    end
    else
      if Time > 15.0 then Back:=true;
  end else
  begin
    Alpha:=Alpha-5;
    X:=X-1;
    if Alpha <= 10 then
      Active:=false;
  end;
end;

end.
