unit uMath;
{
  Original Code by XProger
  xproger@list.ru
}
interface
type
  TVector2D = record
    X,Y:single;
  end;

  TVector = record
    X,Y,Z:single;
  end;

  TFace = array[0..2] of word;

  // 2D ...
  function ArcTan2(const Y,X:single):single;
  function ArcCos(const X:single):single;
  function Vec2D(X,Y:single):TVector2D;
  function Vec2D_Dist(V1,V2:TVector2D):single;
  // 3D ...
  function Vec(X,Y,Z:single):TVector;
  function Vec_Add(V1,V2:TVector):TVector;
  function Vec_Sub(V1,V2:TVector):TVector;
  function Vec_Mult(V1:TVector;m:single):TVector;
  function Vec_Div(V:TVector;d:single):TVector;
  function Vec_Dist(var V1,V2:TVector):single;
  function Vec_Length(V:TVector):single;
  function Vec_Norm(V:TVector):TVector;
  function Vec_Cross(V1,V2:TVector):TVector;
 // Plane
  function Plane_Normal(V1,V2,V3:TVector):TVector;

 // Simple
  function Max(const x,y:single):single;
  function Min(const x,y:single):single;

implementation

  function ArcTan2(const Y,X:single):single;
  asm
    FLD     Y
    FLD     X
    FPATAN
    FWAIT
  end;

  function ArcCos(const X:single):single;
  begin
    if abs(X) > 1 then
      Result:=0
    else
      Result:=ArcTan2(Sqrt(1 - X * X),X);
  end;

  function Vec2D(X,Y:single):TVector2D;
  begin
    Result.X:=X;
    Result.Y:=Y;
  end;

  function Vec2D_Dist(V1,V2:TVector2D):single;
  begin
    Result:=Sqrt(Sqr(V1.X-V2.X)+Sqr(V1.Y-V2.Y));
  end;

  function Vec(X,Y,Z:single):TVector;
  begin
    Result.X:=X;
    Result.Y:=Y;
    Result.Z:=Z;
  end;

  function Vec_Add(V1,V2:TVector):TVector;
  begin
    Result.X:=V1.X + V2.X;
    Result.Y:=V1.Y + V2.Y;
    Result.Z:=V1.Z + V2.Z;
  end;

  function Vec_Sub(V1,V2:TVector):TVector;
  begin
    Result.X:=V1.X - V2.X;
    Result.Y:=V1.Y - V2.Y;
    Result.Z:=V1.Z - V2.Z;
  end;

  function Vec_Mult(V1:TVector;m:single):TVector;
  begin
    Result.X:=V1.X * m;
    Result.Y:=V1.Y * m;
    Result.Z:=V1.Z * m;
  end;

  function Vec_Div(V:TVector;d:single):TVector;
  begin
    if d <> 0 then
      Result:=Vec(V.X/d, V.Y/d,V.Z/d)
    else
      Result:=V;
  end;

  function Vec_Dist(var V1,V2:TVector):single;
  begin
    Result:=sqrt(sqr(V2.X - V1.X)+
                 sqr(V2.Y - V1.Y)+
                 sqr(V2.Z - V1.Z));
  end;

  function Vec_Length(V:TVector):single;
  begin
    Result:=sqrt(sqr(V.X) + sqr(V.Y) + sqr(V.Z));
  end;

  function Vec_Norm(V:TVector):TVector;
  var
    l:single;
  begin
    l:=Vec_Length(V);
    if l <> 0 then
      Result:=Vec_Mult(V,1/l)
    else
      Result:=V;
  end;

  function Vec_Cross(V1,V2:TVector):TVector;
  begin
    Result.X := V1.Y * V2.Z - V1.Z * V2.Y;
    Result.Y := V1.Z * V2.X - V1.X * V2.Z;
    Result.Z := V1.X * V2.Y - V1.Y * V2.X;
  end;

 // Plane
  function Plane_Normal(V1,V2,V3:TVector):TVector;
  begin
    v1:=Vec_Sub(v2, v1);
    v2:=Vec_Sub(v3, v2);
    v3:=Vec_Cross(v1, v2);
    Result:=Vec_Norm(v3);
  end;

  function Max(const x,y:single):single;
  begin
    if x > y then Result:=x else Result:=y;
  end;

  function Min(const x,y:single):single;
  begin
    if x < y then Result:=x else Result:=y;
  end;
  
end.
