unit uParticles;

interface
uses sNail,Windows,OpenGL,uGameObj;
const
  PART_BOOM = 1;
  PART_FIRE = 2;
type
  TPartObject = record
    X,Y:single;
    Life,Speed,Angle:single;
    Alpha:byte;
  end;

  TPartEmitter = class
  protected
    Active:boolean;
    X,Y:single;
    Color:integer;
    Objs:array of TPartObject;
    Count:word;
    Immortal:boolean;
  public
    constructor Create(dX,dY:single;Clr:integer);virtual;
    procedure SetCount(const nCount:word);
    procedure Update;
    procedure Update_part(ID:byte);virtual;
    procedure Render;virtual;
    procedure Reset(ID:byte);virtual;
  end;

  TPartBoom = class(TPartEmitter)
  public
    constructor Create(dX,dY:single;Clr:integer);override;
    procedure Update_part(ID:byte);override;
  end;

  TPartFire = class(TPartEmitter)
  public
    constructor Create(dX,dY:single;Clr:integer);override;
    procedure Update_part(ID:byte);override;
    procedure Reset(ID:byte);override;
  end;  

  TPartEngine = class
  public
    Objs:array[0..31] of TPartEmitter;

    constructor Create;
    procedure Add(PType:byte; X,Y:single;Color:integer);
    procedure Delete(ID:integer);
    procedure Update;
    procedure Render;
  end;

var
  tex_part:cardinal;

implementation
uses uGameEngine;
{ TParticleEmitter }

procedure TPartEmitter.SetCount(const nCount: word);
begin
  SetLength(Objs,nCount);
  Count:=High(objs);
end;

constructor TPartEmitter.Create(dX, dY: single;Clr:integer);
begin
  inherited Create;
  X:=dX;
  Y:=dY;
  Color:=Clr;
  Active:=true;
end;

procedure TPartEmitter.Render;
var
  i:byte;
begin
  gl_TexEnable(tex_part);
  for i:=0 to Count-1 do
    with Objs[i] do
      if Life > 0 then
      begin
        gl_SetColor(Color,Alpha);
        glPushMatrix;
          gl_RenderTex(X-8,Y-8,16,16,0);
        glPopMatrix;
      end;
  gl_TexDisable;
end;

procedure TPartEmitter.Reset(ID:byte);
begin
end;

procedure TPartEmitter.Update_part(ID: byte);
begin
end;

procedure TPartEmitter.Update;
var
  i:byte;
begin
  inherited;
  for i:=0 to Count-1 do
    with Objs[i] do
    begin
      if Life > 0 then
      begin
        Life:=Life - 1.0;
        Update_part(i);
      end else
        if Immortal then
          Reset(i)
        else
          Active:=false;
    end;
end;

{ TPartEngine ------------------------------------------}

constructor TPartEngine.Create;
begin
  ZeroMemory(@Objs,sizeof(Objs));
end;

procedure TPartEngine.Add(PType: byte; X, Y: single;Color:integer);
var
  i,index:shortint;
begin
  index:=-1;
  for i:=0 to High(Objs) do
    if Objs[i]  = nil then
    begin
      index:=i;
      break;
    end;

  if index > -1 then
    case PType of
      PART_BOOM:Objs[index]:=TPartBoom.Create(X,Y,Color);
      PART_FIRE:Objs[index]:=TPartFire.Create(X,Y,Color);
    end;
end;

procedure TPartEngine.Delete(id: integer);
begin
  Objs[id].Free;
  Objs[id]:=nil;
end;

procedure TPartEngine.Render;
var
  i:word;
begin
  for i:=0 to High(Objs) do
    if (Objs[i] <> nil) then
      Objs[i].Render;
end;

procedure TPartEngine.Update;
var
  i:word;
begin
  for i:=0 to High(objs) do
    if Objs[i] <> nil then
      if not Objs[i].Active then
        Delete(i)
      else
        Objs[i].Update;
end;

{ TBoom }

constructor TPartBoom.Create(dX, dY: single; Clr:integer);
var
  i:byte;
begin
  inherited;
  SetCount(8);
  Immortal:=false;

  for i:=0 to Count - 1 do
    with Objs[i] do
    begin
      X:=Self.X;
      Y:=Self.Y;
      Life:=12;
      Alpha:=255;
      Angle:=random*pi*2;
      Speed:=2.0;
    end;
end;

procedure TPartBoom.Update_part(ID: byte);
begin
  inherited;
  with Objs[ID] do
  begin
    Speed:=Speed+0.3;
    X:=X-sin(Angle)*Speed;
    Y:=Y-cos(Angle)*Speed;
    Alpha:=Alpha - 15;
  end;
end;

{ TPartFire }

constructor TPartFire.Create(dX, dY: single; Clr: integer);
var
  i:byte;
begin
  inherited;
  SetCount(15);
  Immortal:=true;
  for i:=0 to High(Objs) do
    Reset(i);
end;

procedure TPartFire.Reset(ID:byte);
begin
  inherited;
  with Objs[ID] do
  begin
    X:=BallPos.X;
    Y:=BallPos.Y;
    Life:=ID;
    Alpha:=255;
    Angle:=random*pi*2;
  end;
end;

procedure TPartFire.Update_part(ID: byte);
begin
  inherited;
  with Objs[ID] do
  begin
    X:=X+sin(Angle);
    Y:=Y+cos(Angle);
    color:=color - 3;
    Alpha:=Alpha-Round(Life*3);
  end;
end;

end.
