#include "Brick.h"
Brick::Brick (Field *field, hgeVector center, float x, float y, int i, int j, float angle=0, const char* textureName): field(field), i(i), j(j), angle(angle) {
    hge=field->getHge();
    coords.x = x;
    coords.y = y;

    refresh (center, angle, 0);
    texture = hge->Texture_Load(textureName);
    ballHitSnd = hge->Effect_Load(BRICK_BALL_HIT_SND_NAME.c_str());

    sprite = new hgeSprite (texture, 0, 0, BRICK_WIDTH, BRICK_HEIGHT);
    sprite->SetHotSpot(BRICK_WIDTH/2, BRICK_HEIGHT/2);

    dying = false;
    timer = 0;
}

Brick::~Brick() {
    hge->Texture_Free(texture);
    delete sprite;
}

void Brick::refresh (hgeVector center, float angle, float dt) {
    this->angle = angle;
    absCoords = coords;
    absCoords.Rotate(angle);
    absCoords+=center;

    if (dying)
        timer-=dt;
    if (timer<0) {
        field->destroyBrick(i, j);
    }
}

void Brick::render () {
    sprite->RenderEx(absCoords.x, absCoords.y, angle);
}

Brick* Brick::collide (Ball *ball) {
    if (ball->isOnRacket()) {
        return this;
    }

    hgeVector ballCoords = ball->getCoords();
    ballCoords-=absCoords;

    ballCoords.Rotate(-angle);

    hgeVector ballSpeed = ball->getSpeed();
    ballSpeed.Rotate(-angle);


    if (abs(ballCoords.x)<BRICK_WIDTH/2+ball->getRad()-1&&abs(ballCoords.y)<BRICK_HEIGHT/2+ball->getRad()-1) {
        if ((abs(ballCoords.x)<BRICK_WIDTH/2) && (ballSpeed.y*ballCoords.y<0)) {
           ballSpeed.y*=-1;
       } else
            if ((abs(ballCoords.y)<BRICK_HEIGHT/2) && (ballSpeed.x*ballCoords.x<0)) {
                ballSpeed.x*=-1;
            } else
                if (ballSpeed.y*ballCoords.y<0 && ballSpeed.x*ballCoords.x<0) {
                    if (abs(ballCoords.x)-BRICK_WIDTH/2>abs(ballCoords.y)-BRICK_HEIGHT/2)
                        ballSpeed.x*=-1;
                    else
                        ballSpeed.y*=-1;
                } else
                    if (ballSpeed.y*ballCoords.y<0) {
                        ballSpeed.y*=-1;
                    } else
                        if (ballSpeed.x*ballCoords.x<0) {
                            ballSpeed.x*=-1;
                        } else
                            return this;

        ballSpeed.Rotate(angle);
        ball->setSpeed(ballSpeed);
        hge->Effect_PlayEx(ballHitSnd);
        onHit();
    }
    return this;
}

void Brick::onHit() {
    field->destroyBrick(i, j);
}

void Brick::die() {

}

void Brick::dieIn(float time) {
    if (!dying) {
        timer = time;
        dying = true;
    }

}

bool Brick::isCountable () {
    return true;
}
