#include "Field.h"
#include "WallBrick.h"
#include "FireBrick.h"


Field::Field (HGE *hge): hge(hge) {
    font = new hgeFont("font1.fnt", true);

    racket = new Racket(this, hge);

    oopsSnd = hge->Effect_Load (OOPS_SND_NAME.c_str());
    restart ();

}

Field::~Field () {
    delete racket;
}

void Field::refresh(float dt) {
    if (levelFinished) {
        if (hge->Input_KeyUp(HGEK_LBUTTON)) {
            lives+=2;
            level++;
            startLevel();
        }
        return;
    }

    if (win||fail) {
        if (hge->Input_KeyUp(HGEK_LBUTTON)) {
            restart();
        }
        return;
    }

    if (hge->Input_KeyUp(HGEK_P)) {
        pause = !pause;
    }

    if (pause) {
        return;
    }
    
    if (hge->Input_GetKeyState(HGEK_N)&&hge->Input_GetKeyState(HGEK_E)&&
       hge->Input_GetKeyState(HGEK_X)&&hge->Input_GetKeyState(HGEK_T)) {
           bricksOnField = 0;
    }
 
    refreshAngle (dt);

    if (balls.size()==0) {
        rebirth();
    }

    racket->refresh(dt);

    vector<Ball*>::iterator iter;

    for (iter = balls.begin(); iter< balls.end(); iter++) {
        (*iter)->refresh(dt);
        racket->collide (*iter);
        if (outOfField((*iter)->getCoords())) {
            delete *iter;
            hge->Effect_PlayEx(oopsSnd);
            balls.erase(iter);
        }
    }
    
    if (hge->Input_KeyUp(HGEK_LBUTTON))
        for (iter = balls.begin(); iter< balls.end(); iter++) {
            (*iter)->fire();
    }

    for (int i=0; i<DIM_Y; i++) {
        for (int j=0; j<DIM_X; j++) {
            if (bricks[i][j]) {
                for (iter = balls.begin(); iter< balls.end(); iter++) {
                    bricks[i][j]->collide (*iter);
                }
                if (bricks[i][j])
                    bricks[i][j]->refresh (center, angle, dt);
            }
        }
    }

    if (bricksOnField<1) {
        score ++;
        if (level<LEVEL_NUMBER-1) {
            levelFinished = true;
            return;
        }
        win = true;
    }

}

void Field::render() {
    racket->render();
    vector<Ball*>::iterator i;

    for (i = balls.begin(); i< balls.end(); i++) {
        (*i)->render();
    }

    for (int i=0; i<DIM_Y; i++) {
        for (int j=0; j<DIM_X; j++) {
            if (bricks[i][j]) {
                bricks[i][j]->render();
            }
        }
    }

    renderFont ();
}

void Field::renderFont () {
    font->printf(10, 10, HGETEXT_LEFT, "Score: %d", score);

    font->printf(790, 10, HGETEXT_RIGHT, "Level: %d", level+1);

    font->printf(790, 30, HGETEXT_RIGHT, "Lives: %d", lives);

    font->printf(10, 480, HGETEXT_LEFT, "C - change \n   control type\nP - pause\nESC - exit");

    if (racket->getControlType()==HOR_CONTROL)
        font->printf(790, 550, HGETEXT_RIGHT, "Control type:\n  horizontal");
    else
        font->printf(790, 550, HGETEXT_RIGHT, "Control type:\n  circular");



    if (levelFinished) {
        font->printf(400, 300, HGETEXT_CENTER, "LEVEL FINISHED\nCLICK TO CONTINUE");
    }

    if (win) {
        font->printf(400, 300, HGETEXT_CENTER, "YOU WIN\nCLICK TO RESTART");
    }

    if (fail) {
        font->printf(400, 300, HGETEXT_CENTER, "YOU FAIL\nCLICK TO RESTART");
    }

    if (pause) {
        font->printf(400, 300, HGETEXT_CENTER, "PAUSE\nPRESS P TO CONTINUE");
    }


}

hgeVector Field::getEdge(float angle) {
    hgeVector v;
    v.x = CENTER_X + FIELD_RAD*sin(angle)*1.3;
    v.y = CENTER_Y + FIELD_RAD*cos(angle);
    return v;
}

bool Field::outOfField (hgeVector coords) {
    return (coords.x<0||coords.y<0||coords.x>800||coords.y>600);
}

bool Field::outOfField (int i, int j) {
    return (i<0||j<0||i>=DIM_Y||j>=DIM_X);
}

void Field::rebirth() {
    if (lives<2) {
        fail=true;
        return;
    }

    lives--;
    birth ();
}

void Field::birth() {
    balls.push_back(new Ball(hge, racket));
}

HGE* Field::getHge () {
    return hge;
}

void Field::destroyBrick (int i, int j) {
    if (outOfField (i, j))
        return;
    if (bricks[i][j]) {
        Brick *b = bricks[i][j];
        bricks[i][j] = 0;
        if (b->isCountable())
            bricksOnField--;

        b->die();
        delete b;


    }
}

void Field::destroyBrick (int i, int j, float time) {
    if (outOfField (i, j))
        return;

    if (bricks[i][j]) {
        bricks[i][j]->dieIn (time);
    }
}

void Field::startLevel () {

    pause = false;
    levelFinished = false;


    vector<Ball*>::iterator iter;

    racket->setAngle (0);
    for (iter = balls.begin(); iter< balls.end(); iter++) {
        delete *iter;
        balls.erase(iter);
    }

    birth();

    bricksOnField = 0;
    angle = levels[level][DIM_Y][0]*M_PI/180;
    startAngle = levels[level][DIM_Y][3]*M_PI/180;
    finishAngle = levels[level][DIM_Y][4]*M_PI/180;
    rotationDir = levels[level][DIM_Y][5];


    center.x = CENTER_X;
    center.y = CENTER_Y;

    float w = BRICK_WIDTH*DIM_X;
    float h = BRICK_HEIGHT*DIM_Y;

    hgeVector v0 (CENTER_X - DIM_X*BRICK_WIDTH, CENTER_Y);

    for (int i=0; i<DIM_Y; i++) {
        for (int j=0; j<DIM_X; j++) {

            if (bricks[i][j])
                delete bricks[i][j];

            bricks[i][j] = 0;
            float x = (j+1/2)*BRICK_WIDTH - w/2;
            float y = (i+1/2)*BRICK_HEIGHT - h/2;


            if (levels[level][i][j]==1) {
                bricks[i][j] = new Brick(this, center, x, y, i, j, 0);
                bricksOnField++;
            }

            if (levels[level][i][j]==11) {
                bricks[i][j] = new Brick(this, center, x, y, i, j, 0, "brick11.png");
                bricksOnField++;
            }

            if (levels[level][i][j]==2) {
                bricks[i][j] = new WallBrick(this, center, x, y, i, j, 0);
            }

            if (levels[level][i][j]==3) {
                bricks[i][j] = new FireBrick(this, center, x, y, i, j, 0);
                bricksOnField++;
            }

        }
    }
}

void Field::refreshAngle (float dt) {
    float maxSpeed = levels[level][DIM_Y][1];
    float minSpeed = levels[level][DIM_Y][2];

    float speed = maxSpeed;

    if (levels[level][DIM_Y][3]!=levels[level][DIM_Y][4]) {
        float c = (finishAngle + startAngle)/2;
        float b = finishAngle - c;

        float x = fabs (angle - c);
        float k = (b-x)/b;

        k=sqrt(sqrt(fabs(k)));

        cout<<k << " " << x << " " << b << " " << c << " " << angle << "\r\n";

        speed = minSpeed + (maxSpeed-minSpeed)*k;

        if (rotationDir>0) {
            if (angle>finishAngle)
                rotationDir*=-1;
        }

        if (rotationDir<0) {
            if (angle<startAngle)
                rotationDir*=-1;
        }
    }

//    cout<<speed;
    angle+=rotationDir*speed*dt/100;
}


void Field::restart() {
    win = false;
    fail= false;
    lives = 5;
    level = 0;
    score = 0;
    startLevel ();
}
