#ifndef FIELD_H_INCLUDED
#define FIELD_H_INCLUDED

class Field;

#include "Ball.h"
#include "Racket.h"
#include "vector.h"
#include "Brick.h"
#include "hgefont.h"
#include <string>


const float CENTER_X = 400;
const float CENTER_Y = 300;
const float FIELD_RAD = 280;

const int DIM_X = 15;
const int DIM_Y = 15;

const std::string OOPS_SND_NAME = "oops.wav";

class Field {
    private:

    hgeFont *font;


    HGE *hge;
    hgeVector center;
    HEFFECT oopsSnd;

    Racket *racket;
    vector<Ball*> balls;

    Brick* bricks[DIM_Y][DIM_X];

    float angle, startAngle, finishAngle, maxSpeed, minSpeed, rotationDir;
    void startLevel ();
    void refreshAngle (float dt);

    int level, score;
    int bricksOnField;
    int lives;

    bool levelFinished, win, fail, pause;

    void renderFont();

    void restart();
//    Brick* bricksArray[][];

    public:

    Field (HGE *hge);
    ~Field ();

    void refresh(float dt);
    void render();

    hgeVector getEdge(float angle);

    bool outOfField (hgeVector coords);
    bool outOfField (int i, int j);
    void rebirth ();
    void birth ();
    HGE *getHge ();
    void destroyBrick (int i, int j);
    void destroyBrick (int i, int j, float time);
};


const int LEVEL_NUMBER=5;

const int levels[LEVEL_NUMBER][DIM_Y+1][DIM_X]= {
    {
        {3, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 3},
        {1, 1, 0, 0, 1, 1, 0, 0, 1, 3, 0, 0, 1, 3, 0},
        {2, 3, 0, 1, 3, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0},
        {2, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1},
        {1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
        {1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 3, 2},
        {0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 2},
        {0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1},
        {1, 3, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
        {1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 3, 0},
        {2, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0},
        {2, 3, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1},
        {1, 0, 1, 0, 1, 3, 1, 0, 1, 3, 1, 0, 1, 0, 1},
        {1, 1, 3, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 3, 2},
        {3, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 3},

        {0, 0, 0, 0, 0, 0}
    },

    {
        {0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0},
        {0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0},
        {0, 0, 0, 0, 2,11, 3,11,11, 2, 0, 0, 0, 0, 0},
        {0, 0, 0, 0, 0,11,11, 3,11, 0, 0, 0, 0, 0, 0},
        {0, 0, 0, 0, 2,11, 3,11,11, 0, 0, 0, 0, 0, 0},
        {0, 0, 0, 0, 0,11,11, 3,11, 2, 0, 0, 0, 0, 0},
        {0, 0, 0, 0, 0,11, 3,11,11, 0, 0, 0, 0, 0, 0},
        {0, 0, 0, 0, 2,11,11, 3,11, 0, 0, 0, 0, 0, 0},
        {0, 0, 0, 0, 0,11, 3,11,11, 2, 0, 0, 0, 0, 0},
        {0, 0, 0, 0, 0,11,11, 3,11, 0, 0, 0, 0, 0, 0},
        {0, 0, 0, 0, 2,11, 3,11,11, 0, 0, 0, 0, 0, 0},
        {0, 0, 0, 0, 0,11,11, 3,11, 2, 0, 0, 0, 0, 0},
        {0, 0, 0, 0, 1, 1, 1, 1, 3, 0, 0, 0, 0, 0, 0},
        {0, 0, 0, 1, 3, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0},
        {0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0},

        {3, 0, 0, 0, 0, 0} //   ,
                        //.  , .  
                        // ,  ,  
    },


    {
        { 2, 2, 2, 2, 2,11, 2,11, 2,11, 2,11, 2, 0, 2},
        { 0, 0, 0, 0, 3, 1, 3, 0, 3, 1, 3, 1, 2, 0, 2},
        { 2, 2, 2, 2, 3, 2, 2, 3, 2, 3, 2, 2, 2, 0, 2},
        {11, 1, 2, 2, 3, 2, 2, 2, 3, 2, 2, 2, 2, 0, 2},
        { 2, 3, 2, 2, 3, 2, 2, 2, 3, 2, 3, 3, 3, 3, 2},
        {11, 1, 3, 2, 2, 3, 2, 3, 2, 3, 2, 2, 2, 1,11},
        { 2, 3, 2, 3, 3, 2, 3, 3, 3, 2, 2, 2, 2, 3, 2},
        {11, 0, 3, 2, 2, 3, 3, 0, 3, 3, 2, 2, 3, 0,11},
        { 2, 3, 2, 2, 2, 2, 3, 3, 3, 2, 3, 3, 2, 3, 2},
        {11, 1, 2, 2, 2, 3, 2, 3, 2, 3, 2, 2, 3, 1,11},
        { 2, 3, 3, 3, 3, 2, 3, 2, 2, 2, 3, 2, 2, 3, 2},
        { 2, 0, 2, 2, 2, 2, 3, 2, 2, 2, 3, 2, 2, 1,11},
        { 2, 0, 2, 2, 2, 3, 2, 3, 2, 2, 3, 2, 2, 2, 2},
        { 2, 0, 2, 1, 3, 1, 3, 0, 3, 1, 3, 0, 0, 0, 0},
        { 2, 0, 2,11, 2,11, 2,11, 2,11, 2, 2, 2, 2, 2},

        {0, 10, 5, -50, 50, 1} //   ,
                        //.  , .  
                        // ,  ,  
    },


    {
        {11, 0, 0, 0, 0, 2, 2, 3, 2, 2, 0, 0, 0, 0,11},
        {11, 3, 0, 2, 2, 0, 0, 0, 0, 0, 2, 2, 0, 3,11},
        {11,11, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2,11,11},
        { 0, 0, 2, 0, 1, 1, 0, 0, 0, 1, 1, 0, 2, 0, 0},
        { 0, 0, 2, 1, 3, 3, 1, 0, 1, 3, 3, 1, 2, 0, 0},
        { 0, 0, 2, 0, 1, 1, 0, 0, 0, 1, 1, 0, 2, 0, 0},
        { 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0},
        { 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0},
        { 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0},
        { 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0},
        { 0, 0, 0, 0, 2, 2, 2, 0, 2, 2, 2, 0, 0, 0, 0},
        { 0, 0, 0, 0, 0, 2, 0, 0, 0, 2, 0, 0, 0, 0, 0},
        { 0, 0, 0, 0, 0, 2,11,11,11, 2, 0, 0, 0, 0, 0},
        { 0, 0, 0, 0, 0, 0,11, 3,11, 0, 0, 0, 0, 0, 0},
        { 0, 0, 0, 0, 0, 0,11,11,11, 0, 0, 0, 0, 0, 0},

        {0, 60, 5, -5, 5, 1} //   ,
                        //.  , .  
                        // ,  ,  
    },

    {
        {1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1},
        {1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0},
        {2, 3, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0},
        {2, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1},
        {1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
        {1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 3, 2},
        {0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 2},
        {0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1},
        {1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1},
        {1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0},
        {2, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0},
        {2, 3, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1},
        {1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1},
        {1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 3, 2},
        {0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2},

        {0, 50, 5, 20, 20, 1}
    }
};

#endif // FIELD_H_INCLUDED
