#include "Racket.h"

Racket::Racket(Field *field, HGE *hge, float angle, float rad): field(field), hge(hge), angle(angle), rad(rad) {
    texture = hge->Texture_Load("racket.png");
    height = START_HEIGHT;
    length = START_LENGTH;

    sprite = new hgeSprite (texture, 0, 0, length, height);
    sprite->SetHotSpot(length/2, height/2);

    pawSnd = hge->Effect_Load("paw.wav");
    controlType = HOR_CONTROL;
}


Racket::~Racket() {
    delete sprite;
    hge->Texture_Free(texture);
    hge->Effect_Free(pawSnd);
}

void Racket::render() {

    sprite->RenderEx (coords.x, coords.y, -angle);
}

void Racket::refresh(float dt) {
    if (hge->Input_KeyUp(HGEK_C)) {
        if (controlType==HOR_CONTROL)
            controlType=CIRC_CONTROL;
        else
            controlType = HOR_CONTROL;
    }

    static bool first = true;
    static float mouseX, mouseY;


    hge->Input_GetMousePos (&mouseX, &mouseY);
    if (first) {
        mouseX = 300;
        mouseY = 300;
    }
    first = false;


    hgeVector v1(sin(angle), cos(angle));

    hgeVector v2(mouseY-300, 300-mouseX);

    float dx = v1.Dot(&v2);
    float da = 0;
    if (controlType==HOR_CONTROL) {
        da = (mouseX-300)*RACKET_SPEED*dt;
    } else {
        da = -dx*dt*RACKET_SPEED;
    }



    if (fabs(da)>MAX_RACKET_SPEED)
        da = MAX_RACKET_SPEED*da/fabs(da);

    angle += da;
    hge->Input_SetMousePos(300, 300);

    float realRad = (FIELD_RAD-height/2)*rad;

    coords = field->getEdge(angle);

    coords*=(coords.Length()-height/2)*rad/coords.Length();
//    coords.x = CENTER_X + realRad*sin(angle);
//    coords.y = CENTER_Y + realRad*cos(angle);
}

float Racket::getAngle () {
    return angle;
}

float Racket::getHeight () {
    return height;
}

hgeVector Racket::getCoords() {
    return coords;
}

void Racket::collide(Ball *ball) {
    if (ball->isOnRacket()) {
        return;
    }

    hgeVector ballCoords = ball->getCoords();
    ballCoords-=coords;

    ballCoords.Rotate(angle);

    hgeVector ballSpeed = ball->getSpeed();
    ballSpeed.Rotate(angle);


    if (abs(ballCoords.x)<length/2+ball->getRad()-1&&abs(ballCoords.y)<height/2+ball->getRad()-1) {

                if (ballSpeed.y*ballCoords.y>=0) {
                    return;
                }

        hgeVector reflSpeed;
        reflSpeed.x = ballCoords.x;
        reflSpeed.y = -height;
        reflSpeed.Normalize();
        reflSpeed.Rotate(-angle);
        ball->setSpeed(reflSpeed);

        ballCoords.y = -(height/2+ball->getRad());
        ballCoords.Rotate(-angle);
        ballCoords+=coords;
        ball->setCoords (ballCoords);
        hge->Effect_PlayEx(pawSnd);
    }

}

void Racket::setAngle (float angle) {
    this->angle = angle;
}

int Racket::getControlType () {
    return controlType;
}
